@@ -7,15 +7,19 @@
 (*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
-(*  $Id: emisc.ml,v 1.2 2005/02/25 17:49:18 maranget Exp $          *)
+(*  $Id: emisc.ml,v 1.3 2006-10-09 08:25:16 maranget Exp $          *)
 (***********************************************************************)
 
+let verbose = ref 0
+
 let basefont = ref 3
 
 let reset () =  basefont := 3
 
-let warning s =
-  Location.print_pos () ;
-  prerr_string "Warning: " ;
-  prerr_endline s
+module OString =
+  struct
+    type t = string
+    let compare = String.compare
+  end
 
+module Strings = Set.Make (OString)
