@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: htmlCommon.ml,v 1.45 2005/01/18 16:25:12 maranget Exp $" 
+let header = "$Id: htmlCommon.ml,v 1.61 2006-10-11 16:42:39 maranget Exp $" 
 
 (* Output function for a strange html model :
      - Text elements can occur anywhere and are given as in latex
@@ -29,10 +29,11 @@ type block =
   | H1 | H2 | H3 | H4 | H5 | H6
   | PRE
   | TABLE | TR | TD
-  | DISPLAY
+  | DISPLAY of bool | DFLOW
   | QUOTE | BLOCKQUOTE
   | DIV
   | UL | OL | DL
+  | LI | DD | DT
   | GROUP | AFTER | DELAY | FORGET
   | INTERN
   | P
@@ -51,7 +52,9 @@ let string_of_block = function
   | TABLE -> "TABLE"
   | TR -> "TR"
   | TD  -> "TD"
-  | DISPLAY -> "DISPLAY"
+  | DISPLAY false -> "DISPLAY"
+  | DISPLAY true -> "DISPLAY (center)"
+  | DFLOW -> "DFLOW"
   | QUOTE -> "QUOTE"
   | BLOCKQUOTE -> "BLOCKQUOTE"
   | DIV -> "DIV"
@@ -65,6 +68,9 @@ let string_of_block = function
   | P     -> "P"
   | NADA  -> "NADA"
   | INTERN -> "INTERN"
+  | LI -> "LI"
+  | DD -> "DD"
+  | DT -> "DT"
   | OTHER s -> s
 
 let block_t = Hashtbl.create 17
@@ -90,7 +96,7 @@ add PRE ;
 add TABLE ;
 add TR ;
 add TD ;
-add DISPLAY ;
+add (DISPLAY false) ;
 add QUOTE ;
 add BLOCKQUOTE ;
 add DIV ;
@@ -112,8 +118,8 @@ add NADA ; ()
 
 
 let failclose s b1 b2=
-  raise (Misc.Close (s^": ``"^string_of_block b1^"'' closes ``"^
-                     string_of_block b2^"''"))
+  raise (Misc.Close (s^": '"^string_of_block b1^"' closes '"^
+                     string_of_block b2^"'"))
 ;;
 
 let find_block s =
@@ -122,16 +128,26 @@ let find_block s =
   | Not_found -> OTHER s
 ;;
 
+let eq_tags t1 t2 = match t1, t2 with
+| DISPLAY _, DISPLAY _ -> true
+| _, _ -> t1=t2
+
 let check_block_closed opentag closetag =
-  if opentag <> closetag && not (opentag = AFTER && closetag = GROUP) then
+  if not (eq_tags opentag closetag) &&
+    not (opentag = AFTER && closetag = GROUP) then
     failclose "html" closetag opentag
 ;;
 
-let display_arg  verbose =
-  if verbose > 1 then
-    "BORDER=1 CELLSPACING=0 CELLPADDING=0"
-  else
-    "CELLSPACING=0 CELLPADDING=0"
+let display_arg centering _verbose =
+  let cl = 
+    if !displayverb then "vdisplay"
+    else "display" in
+  let cl =
+    if centering then
+      cl^(if  !displayverb then " vdcenter" else " dcenter")
+    else cl in
+  let arg = "CLASS=\""^cl^"\"" in
+  arg
 ;;
 
 (* output globals *)
@@ -229,20 +245,17 @@ let push_out s (a,b,c) = push s (Normal (a,b,c))
 ;;
 
 let pretty_stack s = Stack.pretty 
-   (function Normal (s,args,_) -> "["^string_of_block s^"]-{"^args^"} "
-   | Freeze _   -> "Freeze ") s
+   (function Normal (s,args,_) -> "["^string_of_block s^"]-{"^args^"}"
+   | Freeze _   -> "Freeze") s
 ;;
 
-let rec pop_out s = match pop s with
+let pop_out s = match pop s with
 | Normal (a,b,c) -> a,b,c
-| Freeze _f      -> raise PopFreeze
-(* begin
-  if !verbose > 2 then begin
-     prerr_string "unfreeze in pop_out" ;
-     pretty_stack !s
-  end ;
-  f () ; pop_out s end
-*)
+| Freeze _       -> raise PopFreeze
+
+and top_out s = match top s with
+| Normal (a,b,c) -> a,b,c
+| Freeze _       -> raise PopFreeze
 ;;
 
 
@@ -267,12 +280,12 @@ and restore_out (a,b) =
   cur_out := a ;
   Stack.restore out_stack b
 
-let pblock () =
-  if Stack.empty out_stack then NADA
-  else
-    match Stack.top out_stack with
-    | Normal (s,_,_) -> s
-    | _ -> NADA
+let pblock () = match Stack.top out_stack with
+| Normal (s,_,_) -> s
+| _ -> NADA
+
+and p2block () = Stack.top2 out_stack
+  
 ;;
 
 let do_put_char c =
@@ -296,14 +309,13 @@ type flags_t = {
     mutable ncols:int;
     mutable empty:bool;
     mutable blank:bool;
-    mutable pending_par: int option;
+    mutable saw_par: bool ;
     mutable vsize:int;
     mutable nrows:int;
     mutable table_vsize:int;
     mutable nitems:int;
     mutable dt:string;
     mutable dcount:string;
-    mutable last_closed:block;
     mutable in_pre:bool;
     mutable insert: (block * string) option;
     mutable insert_attr: (block * string) option;
@@ -331,7 +343,7 @@ and close_top n out = match  out.top with
 let debug_attr stderr = function
   | None -> Printf.fprintf stderr "None"
   | Some (tag,attr) ->
-      Printf.fprintf stderr "``%s'' ``%s''"
+      Printf.fprintf stderr "'%s' '%s'"
         (string_of_block tag) attr
 
 let debug_flags f =
@@ -345,14 +357,13 @@ let flags = {
   in_math = false;
   empty = true;
   blank = true;
-  pending_par = None;
+  saw_par = false ;
   vsize = 0;
   nrows = 0;
   table_vsize = 0;
   nitems = 0;
   dt = "";
   dcount = "";
-  last_closed = NADA;
   in_pre = false;
   insert = None;
   insert_attr = None;
@@ -364,14 +375,13 @@ let copy_flags {
   in_math = in_math;
   empty = empty;
   blank = blank;
-  pending_par = pending_par;
+  saw_par = saw_par ;
   vsize = vsize;
   nrows = nrows;
   table_vsize = table_vsize;
   nitems = nitems;
   dt = dt;
   dcount = dcount;
-  last_closed = last_closed;
   in_pre = in_pre;
   insert = insert;
   insert_attr = insert_attr;
@@ -381,14 +391,13 @@ let copy_flags {
   in_math = in_math;
   empty = empty;
   blank = blank;
-  pending_par = pending_par;
+  saw_par = saw_par ;
   vsize = vsize;
   nrows = nrows;
   table_vsize = table_vsize;
   nitems = nitems;
   dt = dt;
   dcount = dcount;
-  last_closed = last_closed;
   in_pre = in_pre;
   insert = insert;
   insert_attr = insert_attr;
@@ -399,14 +408,13 @@ and set_flags f {
   in_math = in_math;
   empty = empty;
   blank = blank;
-  pending_par = pending_par;
+  saw_par = saw_par ;
   vsize = vsize;
   nrows = nrows;
   table_vsize = table_vsize;
   nitems = nitems;
   dt = dt;
   dcount = dcount;
-  last_closed = last_closed;
   in_pre = in_pre;
   insert = insert;
   insert_attr = insert_attr;
@@ -416,14 +424,13 @@ and set_flags f {
   f.in_math <- in_math;
   f.empty <- empty;
   f.blank <- blank;
-  f.pending_par <- pending_par;
+  f.saw_par <- saw_par ;
   f.vsize <- vsize;
   f.nrows <- nrows;
   f.table_vsize <- table_vsize;
   f.nitems <- nitems;
   f.dt <- dt;
   f.dcount <- dcount;
-  f.last_closed <- last_closed;
   f.in_pre <- in_pre;
   f.insert <- insert ;
   f.insert_attr <- insert_attr ;
@@ -439,7 +446,7 @@ type stack_t = {
   s_ncols : int Stack.t ;
   s_empty : bool Stack.t ;
   s_blank : bool Stack.t ;
-  s_pending_par : int option Stack.t ;
+  s_saw_par : bool Stack.t ;
   s_vsize : int Stack.t ;
   s_nrows : int Stack.t ;
   s_table_vsize : int Stack.t ;
@@ -460,7 +467,7 @@ let stacks = {
   s_ncols = Stack.create "ncols" ;
   s_empty = Stack.create_init "empty" false;
   s_blank = Stack.create_init "blank" false ;
-  s_pending_par = Stack.create "pending_par" ;
+  s_saw_par = Stack.create "saw_par" ;
   s_vsize = Stack.create "vsize" ;
   s_nrows = Stack.create_init "nrows" 0 ;
   s_table_vsize = Stack.create_init "table_vsize" 0 ;
@@ -480,7 +487,7 @@ type saved_stacks = {
   ss_ncols : int Stack.saved ;
   ss_empty : bool Stack.saved ;
   ss_blank : bool Stack.saved ;
-  ss_pending_par : int option Stack.saved ;
+  ss_saw_par : bool Stack.saved ;
   ss_vsize : int Stack.saved ;
   ss_nrows : int Stack.saved ;
   ss_table_vsize : int Stack.saved ;
@@ -502,7 +509,7 @@ let save_stacks () =
   ss_ncols = Stack.save stacks.s_ncols ;
   ss_empty = Stack.save stacks.s_empty ;
   ss_blank = Stack.save stacks.s_blank ;
-  ss_pending_par = Stack.save stacks.s_pending_par ;
+  ss_saw_par = Stack.save stacks.s_saw_par ;
   ss_vsize = Stack.save stacks.s_vsize ;
   ss_nrows = Stack.save stacks.s_nrows ;
   ss_table_vsize = Stack.save stacks.s_table_vsize ;
@@ -523,7 +530,7 @@ and restore_stacks
   ss_ncols = saved_ncols ;
   ss_empty = saved_empty ;
   ss_blank = saved_blank ;
-  ss_pending_par = saved_pending_par ;
+  ss_saw_par = saved_saw_par ;
   ss_vsize = saved_vsize ;
   ss_nrows = saved_nrows ;
   ss_table_vsize = saved_table_vsize ;
@@ -541,7 +548,7 @@ and restore_stacks
   Stack.restore stacks.s_ncols saved_ncols ;
   Stack.restore stacks.s_empty saved_empty ;
   Stack.restore stacks.s_blank saved_blank ;
-  Stack.restore stacks.s_pending_par saved_pending_par ;
+  Stack.restore stacks.s_saw_par saved_saw_par ;
   Stack.restore stacks.s_vsize saved_vsize ;
   Stack.restore stacks.s_nrows saved_nrows ;
   Stack.restore stacks.s_table_vsize saved_table_vsize ;
@@ -569,7 +576,7 @@ let check_stacks () = match stacks with
   s_ncols = s_ncols ;
   s_empty = s_empty ;
   s_blank = s_blank ;
-  s_pending_par = s_pending_par ;
+  s_saw_par = s_saw_par ;
   s_vsize = s_vsize ;
   s_nrows = s_nrows ;
   s_table_vsize = s_table_vsize ;
@@ -587,7 +594,7 @@ let check_stacks () = match stacks with
   check_stack s_ncols ;
   check_stack s_empty ;
   check_stack s_blank ;
-  check_stack s_pending_par ;
+  check_stack s_saw_par ;
   check_stack s_vsize ;
   check_stack s_nrows ;
   check_stack s_table_vsize ;
@@ -637,85 +644,10 @@ let is_list = function
 | _ -> false
 ;;
 
-let string_of_par = function
-  | Some i -> "+"^string_of_int i
-  | None   -> "-"
-
-let par_val last now n =
-  let r = 
-    if is_list last then begin
-      if is_list now then 1 else 0
-    end
-    else if last = P then
-      0
-    else if
-      is_header last || last = PRE || last = BLOCKQUOTE
-    then n-1
-    else if last = TABLE then n
-    else n+1 in
-  if !verbose > 2 then
-    Printf.fprintf stderr
-      "par_val last=%s, now=%s, r=%d\n"
-      (string_of_block last) 
-      (string_of_block now) r ;
-  r
-;;
-
-let par  = function
-  | Some n as p ->
-      flags.pending_par <- p ;
-      if !verbose > 2 then
-        prerr_endline
-          ("par: last_close="^ string_of_block flags.last_closed^
-           " r="^string_of_int n)
-  | _ -> ()
-;;
-
-let flush_par n =
-  flags.pending_par <- None ;
-  for i = 1 to n do
-    do_put "<BR>\n"
-  done ;
-  if n <= 0 then do_put_char '\n' ;
-  if !verbose > 2 then
-     prerr_endline
-       ("flush_par: last_closed="^ string_of_block flags.last_closed^
-       " p="^string_of_int n);
-  flags.vsize <- flags.vsize + n;
-  flags.last_closed <- NADA
-;;
-
-type t_try = Wait of block | Now
-let string_of_wait = function
-  | Wait b -> "(Wait "^string_of_block b^")"
-  | Now    -> "Now"
-
-let try_flush_par block = match block with
-| Wait GROUP -> ()
-| _ ->  match flags.pending_par with
-  | Some n ->
-      flush_par
-        (match block with
-        | Wait b -> par_val b NADA n
-        | _ -> par_val NADA NADA n)
-  | _      -> ()
-
-
 let string_of_into = function
   | Some n -> "+"^string_of_int n
   | None -> "-"
 
-let forget_par () =
-  let r = flags.pending_par in
-  if !verbose > 2 then
-    prerr_endline
-      ("forget_par: last_close="^ string_of_block flags.last_closed^
-       " r="^string_of_into r) ;  
-  flags.pending_par <- None ;
-  r
-;;
-
-
 
 (* styles *)
 
@@ -978,14 +910,7 @@ let do_open_mods () =
 
 
   
-let do_pending () =  
-  begin match flags.pending_par with
-  | Some n ->
-      flush_par (par_val flags.last_closed (pblock()) n)
-  | _ -> ()
-  end ;
-  flags.last_closed <- NADA ;
-  do_open_mods ()
+let do_pending () =  do_open_mods ()
 ;;
 
 
@@ -1131,7 +1056,7 @@ let erase_mods ms =
 let open_mod  m =
   if not !cur_out.nostyle then begin
     if !verbose > 3 then begin
-      prerr_endline ("open_mod: "^pretty_text m^" ok="^sbool (ok_mod m)) ;
+      prerr_endline ("=> open_mod: "^pretty_text m^" ok="^sbool (ok_mod m)) ;
       pretty_cur !cur_out
     end ;
     begin match m with
@@ -1144,7 +1069,11 @@ let open_mod  m =
         if ok_mod m then begin
           !cur_out.pending <- m :: !cur_out.pending
         end
-    end
+    end ;
+    if !verbose > 3 then begin
+      prerr_endline ("<= open_mod: "^pretty_text m) ;
+      pretty_cur !cur_out
+    end ;
   end
 ;;
 
@@ -1167,55 +1096,64 @@ let pstart = function
   | _ -> false
 ;;
 
+let transmit_par s = match s with
+| GROUP|AFTER|INTERN|DFLOW|P|LI -> false
+| _ -> true
+
 let is_group = function
   | GROUP -> true
   | _ -> false
 
+and transmit_par_or_par = function
+  | P -> true
+  | s -> transmit_par s
+
 and is_pre = function
   | PRE -> true
   | _ -> false
 
 let rec do_try_open_block s =  
   if !verbose > 2 then
-    prerr_flags ("=> try open ``"^string_of_block s^"''");  
-  if s=DISPLAY then begin
-    do_try_open_block TABLE ;
-    do_try_open_block TR  ;
-  end else begin
-    push stacks.s_empty flags.empty ; push stacks.s_blank flags.blank ;
-    push stacks.s_insert flags.insert ;
-    flags.empty <- true ; flags.blank <- true ;
-    flags.insert <- None ;
-    begin match s with
-    | PRE -> flags.in_pre <- true (* No stack, cannot nest *)
-    | TABLE ->
-      push stacks.s_table_vsize flags.table_vsize ;
-      push stacks.s_vsize flags.vsize ;
-      push stacks.s_nrows flags.nrows ;
-      flags.table_vsize <- 0 ;
-      flags.vsize <- 0 ;
-      flags.nrows <- 0
-    |  TR ->
-      flags.vsize <- 1
-    |  TD ->
-      push stacks.s_vsize flags.vsize ;
-      flags.vsize <- 1
-    | _ ->
-        if is_list s then begin
-          push stacks.s_nitems flags.nitems;
-          flags.nitems <- 0 ;
-          if s = DL then begin
-            push stacks.s_dt flags.dt ;
-            push stacks.s_dcount flags.dcount;
-            flags.dt <- "";
-            flags.dcount <- ""
+    prerr_flags ("=> try open '"^string_of_block s^"'");  
+  begin match s with
+  | DISPLAY _ ->
+      do_try_open_block TABLE ;
+      do_try_open_block TR
+  | _  ->
+      push stacks.s_empty flags.empty ; push stacks.s_blank flags.blank ;
+      push stacks.s_insert flags.insert ; push stacks.s_saw_par flags.saw_par ;
+      flags.empty <- true ; flags.blank <- true ;
+      flags.insert <- None ; flags.saw_par <- false ;
+      begin match s with
+      | PRE -> flags.in_pre <- true (* No stack, cannot nest *)
+      | TABLE ->
+	  push stacks.s_table_vsize flags.table_vsize ;
+	  push stacks.s_vsize flags.vsize ;
+	  push stacks.s_nrows flags.nrows ;
+	  flags.table_vsize <- 0 ;
+	  flags.vsize <- 0 ;
+	  flags.nrows <- 0
+      |  TR ->
+	  flags.vsize <- 1
+      |  TD ->
+	  push stacks.s_vsize flags.vsize ;
+	  flags.vsize <- 1
+      | _ ->
+          if is_list s then begin
+            push stacks.s_nitems flags.nitems;
+            flags.nitems <- 0 ;
+            if s = DL then begin
+              push stacks.s_dt flags.dt ;
+              push stacks.s_dcount flags.dcount;
+              flags.dt <- "";
+              flags.dcount <- ""
+            end
           end
-        end
-    end
+      end
   end ;
   if !verbose > 2 then
-    prerr_flags ("<= try open ``"^string_of_block s^"''")
-      ;;
+    prerr_flags ("<= try open '"^string_of_block s^"'")
+;;
 
 let try_open_block s _ =
   push stacks.s_insert_attr flags.insert_attr ;
@@ -1226,8 +1164,6 @@ let try_open_block s _ =
   do_try_open_block s
 
 let do_do_open_block s args =
-  if s = TR || is_header s then
-    do_put "\n";
   do_put_char '<' ;
   do_put (string_of_block s) ;
   if args <> "" then begin
@@ -1238,17 +1174,17 @@ let do_do_open_block s args =
 
   
 let rec do_open_block insert s args = match s with
-| GROUP|DELAY|FORGET|AFTER|INTERN ->
+| GROUP|DELAY|FORGET|AFTER|INTERN|DFLOW ->
    begin match insert with
    | Some (tag,iargs) -> do_do_open_block tag iargs
    | _ -> ()
    end
-| DISPLAY ->
-   do_open_block insert TABLE (display_arg !verbose) ;
+| DISPLAY centering ->   
+   do_open_block insert TABLE (display_arg centering !verbose) ;
    do_open_block None TR args
 | _  -> begin match insert with
   | Some (tag,iargs) ->
-      if is_list s || s = TABLE then begin
+      if is_list s || s = TABLE || s = P then begin
         do_do_open_block tag iargs ;
         do_do_open_block s args
       end else begin
@@ -1260,45 +1196,49 @@ end
 
 let rec do_try_close_block s =
   if !verbose > 2 then
-    prerr_flags ("=> try close ``"^string_of_block s^"''") ;
-  if s = DISPLAY then begin
-    do_try_close_block TR ;
-    do_try_close_block TABLE
-  end else begin
-    let ehere = flags.empty and ethere = pop  stacks.s_empty in
-    flags.empty <- (ehere && ethere) ;
-    let bhere = flags.blank and bthere = pop  stacks.s_blank in
-    flags.blank <- (bhere && bthere) ;
-    flags.insert <- pop  stacks.s_insert ;
-    begin match s with 
-    | PRE   -> flags.in_pre <- false (* PRE cannot nest *)
-    | TABLE ->
-      let p_vsize = pop stacks.s_vsize in
-      flags.vsize <- max
-       (flags.table_vsize + (flags.nrows)/3) p_vsize ;
-      flags.nrows <- pop  stacks.s_nrows ;
-      flags.table_vsize <- pop stacks.s_table_vsize
-    |  TR ->
-        if ehere then begin
-          flags.vsize <- 0
-        end ;
-        flags.table_vsize <- flags.table_vsize + flags.vsize;
-        if not ehere then flags.nrows <- flags.nrows + 1
-    | TD ->
-        let p_vsize = pop stacks.s_vsize in
-        flags.vsize <- max p_vsize flags.vsize
-    | _ ->
-        if is_list s then begin
-          flags.nitems <- pop stacks.s_nitems ;
-          if s = DL then begin
-            flags.dt <- pop stacks.s_dt ;
-            flags.dcount <- pop  stacks.s_dcount
+    prerr_flags ("=> try close '"^string_of_block s^"'") ;
+  begin match s with
+  | DISPLAY _ ->
+      do_try_close_block TR ;
+      do_try_close_block TABLE
+  | _ ->
+      let ehere = flags.empty and ethere = pop  stacks.s_empty in
+      flags.empty <- (ehere && ethere) ;
+      let bhere = flags.blank and bthere = pop  stacks.s_blank in
+      flags.blank <- (bhere && bthere) ;
+      flags.insert <- pop  stacks.s_insert ;
+      flags.saw_par <- pop stacks.s_saw_par ;
+      begin match s with 
+      | PRE   -> flags.in_pre <- false (* PRE cannot nest *)
+      | TABLE ->
+	  let p_vsize = pop stacks.s_vsize in
+	  flags.vsize <-
+	     max
+	       (flags.table_vsize + (flags.nrows+1)/3)
+	       p_vsize ;
+	  flags.nrows <- pop  stacks.s_nrows ;
+	  flags.table_vsize <- pop stacks.s_table_vsize
+      |  TR ->
+          if ehere then begin
+            flags.vsize <- 0
+          end ;
+          flags.table_vsize <- flags.table_vsize + flags.vsize;
+          if not ehere then flags.nrows <- flags.nrows + 1
+      | TD ->
+          let p_vsize = pop stacks.s_vsize in
+          flags.vsize <- max p_vsize flags.vsize
+      | _ ->
+          if is_list s then begin
+            flags.nitems <- pop stacks.s_nitems ;
+            if s = DL then begin
+              flags.dt <- pop stacks.s_dt ;
+              flags.dcount <- pop  stacks.s_dcount
+            end
           end
-        end
-    end
+      end
   end ;
   if !verbose > 2 then
-    prerr_flags ("<= try close ``"^string_of_block s^"''")
+    prerr_flags ("<= try close '"^string_of_block s^"'")
 
 let try_close_block s =
   begin match flags.insert_attr with
@@ -1314,20 +1254,20 @@ let do_do_close_block s =
   do_put "</" ;
   do_put (string_of_block s) ;
   do_put_char '>' ;
-  match s with TD -> do_put_char '\n' | _ -> ()
+  match s with TR -> do_put_char '\n' | _ -> ()
 
 let rec do_close_block insert s = match s with
-|  GROUP|DELAY|FORGET|AFTER|INTERN -> 
+| GROUP|DELAY|FORGET|AFTER|INTERN|DFLOW -> 
    begin match insert with
    | Some (tag,_) -> do_do_close_block tag
    | _ -> ()
    end
-| DISPLAY ->
+| DISPLAY _ ->
     do_close_block None TR ;
     do_close_block insert TABLE
 | s  -> begin match insert with
   | Some (tag,_) ->
-      if is_list s || s = TABLE then begin
+      if is_list s || s = TABLE || s = P then begin
         do_do_close_block s;
         do_do_close_block tag
       end else begin
@@ -1341,36 +1281,40 @@ let check_empty () = flags.empty
 
 and make_empty () =
   flags.empty <- true ; flags.blank <- true ;
+  flags.table_inside <- false ;
   !cur_out.top <- NotMe ;
   !cur_out.pending <-  to_pending !cur_out.pending !cur_out.active ;
   !cur_out.active <- []  
-;;
+
+let check_blank () = flags.blank
+
+let no_check () = false
 
 let rec open_top_styles = function
   | NotMe|Insert (_,_) -> (* Real block, inserted block *)
-        begin match !cur_out.top with
-        | Nothing tops ->
-            let mods =
-              to_pending !cur_out.pending !cur_out.active @
-              to_pending tops.top_pending tops.top_active in
-            assert (!cur_out.active=[]) ;
-            close_active_mods tops.top_active ;
-           !cur_out.top <- Closed (tops,Out.get_pos !cur_out.out);
-            Some mods
-        | Activate tops ->
-            !cur_out.top <- ActivateClosed tops ;
-            let mods =
-              to_pending !cur_out.pending !cur_out.active @
-              to_pending tops.top_pending tops.top_active in
-            close_active_mods !cur_out.active ;
-            close_active_mods (activate "open_top_styles" tops.top_pending) ;
-            close_active_mods tops.top_active ;
-            Some mods
-        | _ ->
-            let mods = to_pending !cur_out.pending !cur_out.active in
-            close_active_mods !cur_out.active ;
-            Some mods
-        end
+      begin match !cur_out.top with
+      | Nothing tops ->
+          let mods =
+            to_pending !cur_out.pending !cur_out.active @
+            to_pending tops.top_pending tops.top_active in
+          assert (!cur_out.active=[]) ;
+          close_active_mods tops.top_active ;
+          !cur_out.top <- Closed (tops,Out.get_pos !cur_out.out);
+          Some mods
+      | Activate tops ->
+          !cur_out.top <- ActivateClosed tops ;
+          let mods =
+            to_pending !cur_out.pending !cur_out.active @
+            to_pending tops.top_pending tops.top_active in
+          close_active_mods !cur_out.active ;
+          close_active_mods (activate "open_top_styles" tops.top_pending) ;
+          close_active_mods tops.top_active ;
+          Some mods
+      | _ ->
+          let mods = to_pending !cur_out.pending !cur_out.active in
+          close_active_mods !cur_out.active ;
+          Some mods
+      end
   | Closed (_,n) -> (* Group that closed top_styles (all of them) *)
       let out = !cur_out in
       let mods = all_to_pending out in
@@ -1386,39 +1330,93 @@ let rec open_top_styles = function
       let r = open_top_styles (Closed (tops,Out.get_pos !cur_out.out)) in
       r
 
-
 let rec force_block s content =
   if !verbose > 2 then begin
     prerr_endline ("=> force_block: ["^string_of_block s^"]");    
+    pretty_stack out_stack ;
     pretty_cur !cur_out
   end ;
-  let was_empty = flags.empty in
+  let pempty = top stacks.s_empty in
   if s = FORGET then begin
     make_empty () ;
-  end else if flags.empty then begin
-    flags.empty <- false; flags.blank <- false ;
-    do_open_mods () ;
-    do_put content
+  end else begin
+    begin match s with
+    | TABLE|DISPLAY _ -> flags.table_inside <- true
+    | _ -> ()
+    end ;
+    if flags.empty then begin
+      flags.empty <- false; flags.blank <- false ;
+      do_open_mods () ;
+      do_put content
+    end ;
+(* check pending display material in DFLOW
+   More precisely
+     * closed block s, has a table inside
+     * previous block is DFLOW, with some pending material (pempty = false)
+   A Freeze can be present, then the previous block is DFLOW
+   Then, we need to flush the pending material...
+*)
+    if not pempty && flags.table_inside then begin
+      let p2 = p2block () in
+      match p2 with
+      |	(Normal (DFLOW,_,_)) | Freeze _ ->
+	  let _,_,pout = top_out out_stack in
+	  if !verbose > 2 then begin
+	    Printf.eprintf "CLOSING: '%s': " (string_of_block s) ;
+	    Out.debug stderr pout.out ;
+	    pretty_stack out_stack ;
+	    prerr_endline ""
+	  end ;
+	  let saved_flags = copy_flags flags in
+	  try_close_block s ;
+	  let a,b,pout = pop_out out_stack in
+	  let saved_out = !cur_out in
+	  cur_out := pout ;
+	  let fo = match p2 with
+	  | Normal (_,_,_) -> force_block DFLOW "" ; None
+	  | Freeze f ->
+	      let _ = pop out_stack in
+	      force_block DFLOW "" ;
+	      Some (f) in
+	  let _,args,_ = top_out out_stack in
+	  force_block TD "" ;
+	  open_block TD args ;
+	  open_block DFLOW "" ;
+	  begin match fo with
+	  | Some f -> push out_stack (Freeze f)
+	  | None -> ()
+	  end ;
+	  push_out out_stack (a,b,!cur_out) ;
+	  try_open_block s b ;
+	  cur_out := saved_out ;
+	  set_flags flags saved_flags ;
+	  flags.ncols <- flags.ncols + 1
+      |	_ -> ()
+    end else if !verbose > 2 && not pempty && flags.table_inside then begin
+      Printf.eprintf "NOT CLOSING: '%s': " (string_of_block s) ;
+      pretty_stack out_stack ;
+      let _,_,pout = top_out out_stack in
+      Out.debug stderr pout.out ;
+      prerr_newline ()
+    end
   end ;
-  if s = TABLE || s=DISPLAY then flags.table_inside <- true;
-(*  if s = PRE then flags.in_pre <- false ; *)
   let true_s = if s = FORGET then pblock() else s in
   let insert = flags.insert
   and insert_attr = flags.insert_attr
   and was_top = !cur_out.top in
-
   do_close_mods () ;
   try_close_block true_s ;
   do_close_block insert true_s ;
   let ps,args,pout = pop_out out_stack in  
   check_block_closed ps true_s ;
+
   let old_out = !cur_out in  
   cur_out := pout ;
   if s = FORGET then free old_out
   else if ps <> DELAY then begin
+
     let mods = open_top_styles was_top in
-    
-    do_open_block insert s
+    do_open_block insert ps
       (match insert_attr with
       | Some (this_tag,attr) when this_tag = s -> args^" "^attr
       | _ -> args) ;
@@ -1429,10 +1427,10 @@ let rec force_block s content =
     | _ -> ()
     end ;
 (*
-    prerr_endline "****** NOW *******" ;
-    pretty_cur !cur_out ;
-    prerr_endline "\n**********" ;
-*)
+  prerr_endline "****** NOW *******" ;
+  pretty_cur !cur_out ;
+  prerr_endline "\n**********" ;
+  *)
     if ps = AFTER then begin
       let f = pop stacks.s_after in
       Out.copy_fun f old_out.out !cur_out.out
@@ -1449,18 +1447,16 @@ let rec force_block s content =
   end else begin (* ps = DELAY *)
     raise (Misc.Fatal ("html: unflushed DELAY"))
   end ;
-  if not was_empty && true_s <> GROUP && true_s <> AFTER then
-    flags.last_closed <- true_s ;
-
+    
   if !verbose > 2 then begin
     prerr_endline ("<= force_block: ["^string_of_block s^"]");    
     pretty_cur !cur_out
-  end ;
+  end
 
-    
+  
 and close_block_loc pred s =
   if !verbose > 2 then
-    prerr_string ("close_block_loc: ``"^string_of_block s^"'' = ");
+    prerr_string ("close_block_loc: '"^string_of_block s^"' = ");
   if not (pred ()) then begin
     if !verbose > 2 then prerr_endline "do it" ;
     force_block s "";
@@ -1472,30 +1468,51 @@ and close_block_loc pred s =
   end
 
 and open_block s args =
- if !verbose > 2 then begin
-   prerr_endline ("=> open_block ``"^string_of_block s^"''"^" arg="^args);
-   pretty_cur !cur_out ;
- end ;
- try_flush_par (Wait s);
+  if !verbose > 2 then begin
+    prerr_endline ("=> open_block '"^string_of_block s^"'"^" arg="^args);
+    pretty_cur !cur_out ;
+  end ;
 
- push_out out_stack (s,args,!cur_out) ;
- cur_out :=
-    begin if is_group s then
-      create_status_from_top !cur_out
-    else
-      create_status_from_scratch
-        !cur_out.nostyle
-        (let cur_mods = all_to_pending !cur_out in
-        if flags.in_pre || is_pre s then filter_pre cur_mods else cur_mods)
-    end ;
- try_open_block s args ;
- if !verbose > 2 then begin
-   prerr_endline ("<= open_block ``"^string_of_block s^"''");
-   pretty_cur !cur_out ;
- end ;
-;;
+  push_out out_stack (s,args,!cur_out) ;
+  cur_out :=
+     begin if false && is_group s then
+       create_status_from_top !cur_out
+     else
+       create_status_from_scratch
+         !cur_out.nostyle
+         (let cur_mods = all_to_pending !cur_out in
+         if flags.in_pre || is_pre s then filter_pre cur_mods else cur_mods)
+     end ;
+  try_open_block s args ;
+
+  if !verbose > 2 then begin
+    prerr_endline ("<= open_block '"^string_of_block s^"'");
+    pretty_cur !cur_out ;
+  end
+
+and close_flow_loc check_empty s =
+  if !verbose > 2 then
+    prerr_endline ("close_flow_loc: "^string_of_block s) ;
+
+  let active  = !cur_out.active
+  and pending = !cur_out.pending in
+  if close_block_loc check_empty s then begin
+    !cur_out.pending <- to_pending pending active ;
+    true
+  end else begin
+    !cur_out.pending <- to_pending pending active ;
+    false
+  end
+
+
+let close_flow s =
+  assert (s <> GROUP) ;
+  if !verbose > 2 then
+    prerr_flags ("=> close_flow '"^string_of_block s^"'");
+  let _ = close_flow_loc check_empty s in
+  if !verbose > 2 then
+    prerr_flags ("<= close_flow '"^string_of_block s^"'")
 
-  
 let insert_block tag arg =
   begin match !cur_out.top with
   | Nothing {top_pending=pending ; top_active=active} ->
@@ -1533,8 +1550,7 @@ let erase_block s =
   end ;
   try_close_block s ;
   let ts,_,tout = pop_out out_stack in
-  if ts <> s && not (s = GROUP && ts = INTERN) then
-    failclose "erase_block" s ts;
+  if ts <> s then failclose "erase_block" s ts;
   free !cur_out ;
   cur_out := tout
 ;;
@@ -1554,19 +1570,25 @@ and open_aftergroup f =
   flags.empty <- false ;
   push stacks.s_after f
 
-and close_group () =
-  match pblock () with
+and close_group () = match pblock () with
   | INTERN -> close_block INTERN
+  | DFLOW ->  close_block DFLOW
   | AFTER  -> force_block AFTER ""
   | _      -> close_block GROUP
+
+and erase_group () = match pblock () with
+  | INTERN -> erase_block INTERN
+  | DFLOW ->  erase_block DFLOW
+  | AFTER  -> erase_block AFTER
+  | _      -> erase_block GROUP
 ;;
 
 
 
 (* output requests  *)
 let is_blank = function
-   ' ' | '\n' -> true
-| _ -> false
+  | ' ' | '\n' -> true
+  | _ -> false
 ;;
 
 let put s =
@@ -1580,12 +1602,10 @@ let put s =
           r := !r && is_blank (String.unsafe_get s i)
         done ;
         !r in
-      let save_last_closed = flags.last_closed in
       do_pending () ;
       flags.empty <- false;
       flags.blank <- s_blank && flags.blank ;
-      do_put s ;
-      if s_blank then flags.last_closed <- save_last_closed
+      do_put s
 ;;
 
 let put_char c =
@@ -1593,13 +1613,11 @@ let put_char c =
   match s with
   | TABLE|TR -> ()
   | _ -> 
-      let save_last_closed = flags.last_closed in
       let c_blank = is_blank c in
       do_pending () ;
       flags.empty <- false;
       flags.blank <- c_blank && flags.blank ;
-      do_put_char c ;
-      if c_blank then flags.last_closed <- save_last_closed
+      do_put_char c
 ;;
 
 
@@ -1617,7 +1635,7 @@ let put_length which  = function
   | Char x -> put (which^string_of_int (Length.font * x))
   | Percent x  -> put (which^"\""^string_of_int x^"%\"")
   | Default    -> ()
-  | No s       -> raise (Misc.Fatal ("No-length ``"^s^"'' in outManager"))
+  | No s       -> raise (Misc.Fatal ("No-length '"^s^"' in outManager"))
 
 let horizontal_line attr width height =
   open_block GROUP "" ;
@@ -1630,56 +1648,12 @@ let horizontal_line attr width height =
   close_block GROUP
 ;;
 
-let skip_column s =
-    put ("&nbsp;</TD><TD NOWRAP ALIGN=center "^s^">")
-;;    
-
-let arrow_in_three_cols dir =
-  if dir then begin
-    put "<TABLE BGCOLOR=black BORDER=0 WIDTH=\"100%\" CELLSPACING=0 " ;
-    put "CELLPADDING=1><TR><TD></TD></TR></TABLE>" ;
-    put "</TD><TD>" ;
-    put "<TABLE BGCOLOR=black BORDER=0 WIDTH=\"100%\" CELLSPACING=0 " ;
-    put ("CELLPADDING=1><TR><TD></TD></TR></TABLE>") ;
-    put "</TD><TD STYLE=\"font-size:smaller\">&#9654;"
-  end
-  else begin
-    put "&#9664;</TD><TD>" ;
-    put "<TABLE BGCOLOR=black BORDER=0 WIDTH=\"100%\" CELLSPACING=0 " ;
-    put "CELLPADDING=1><TR><TD></TD></TR></TABLE>" ;
-    put "</TD><TD>" ;
-    put "<TABLE BGCOLOR=black BORDER=0 WIDTH=\"100%\" CELLSPACING=0 " ;
-    put ("CELLPADDING=1><TR><TD></TD></TR></TABLE>")
-  end 
+let line_in_table () =
+(*  put "<DIV CLASS=\"hbar\"></DIV>" ; *)
+  put "<HR CLASS=\"hbar\">" ;
+  flags.vsize <- flags.vsize - 1
 ;;
 
-
-let arrow_in_table h dir =
-  let pad = (h-1)/2 in
-  if dir then begin
-    put "<TABLE BORDER=0 WIDTH=\"100%\" CELLSPACING=0 CELLPADDING=0>" ;
-    put "<TR><TD>" ;
-    put "<TABLE BGCOLOR=black BORDER=0 WIDTH=\"100%\" CELLSPACING=0 CELLPADDING=" ;
-    put (string_of_int pad) ;
-    put "><TR><TD></TD></TR></TABLE>" ;
-    put "</TD><TD>&gt</TD></TR></TABLE>"
-  end
-  else begin
-    put "<TABLE BORDER=1 WIDTH=\"100%\" CELLSPACING=0 CELLPADDING=0>" ;
-    put "<TR><TD>&lt</TD><TD>" ;
-    put "<TABLE BGCOLOR=black BORDER=1 WIDTH=\"100%\" CELLSPACING=0 CELLPADDING=" ;
-    put (string_of_int pad) ;
-    put "><TR><TD></TD></TR></TABLE>" ;
-    put "</TD></TR></TABLE>"
-  end 
-;;
-
-let line_in_table h =
-  let pad = (h-1)/2 in
-  put "<TABLE BORDER=0 WIDTH=\"100%\" CELLSPACING=0 CELLPADDING=" ;
-  put (string_of_int pad) ;
-  put "><TR><TD></TD></TR></TABLE>"
-
 let freeze f =
   push out_stack (Freeze f) ;
   if !verbose > 2 then begin
@@ -1721,30 +1695,6 @@ and try_close_display () =
 ;;
 
 
-let close_flow_loc s =
-  if !verbose > 2 then
-    prerr_endline ("close_flow_loc: "^string_of_block s) ;
-
-  let active  = !cur_out.active
-  and pending = !cur_out.pending in
-  if close_block_loc check_empty s then begin
-    !cur_out.pending <- to_pending pending active ;
-    true
-  end else begin
-    !cur_out.pending <- to_pending pending active ;
-    false
-  end
-
-
-let close_flow s =
-  assert (s <> GROUP) ;
-  if !verbose > 2 then
-    prerr_flags ("=> close_flow ``"^string_of_block s^"''");
-  let _ = close_flow_loc s in
-  if !verbose > 2 then
-    prerr_flags ("<= close_flow ``"^string_of_block s^"''")
-
-
 
 let get_block s args =
   if !verbose > 2 then begin
@@ -1782,7 +1732,6 @@ let hidden_to_string f =
   prerr_endline "" ;
 *)
   let old_flags = copy_flags flags in
-  let _ = forget_par () in
   open_block INTERN "" ;
   f () ;
   do_close_mods () ;
