@@ -7,7 +7,7 @@
 (*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
-(*  $Id: htmllex.mli,v 1.4 2001/05/28 17:28:55 maranget Exp $          *)
+(*  $Id: htmllex.mli,v 1.6 2006-10-09 08:25:16 maranget Exp $          *)
 (***********************************************************************)
 exception Error of string
 
@@ -16,4 +16,5 @@ val to_string : Lexeme.token -> string
 val cost : Lexeme.style -> int * int
 val reset : unit -> unit
 val next_token : Lexing.lexbuf -> Lexeme.token
-
+val styles : Lexing.lexbuf -> Css.id list
+val classes : Lexing.lexbuf -> Emisc.Strings.t
