@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: index.ml,v 1.42 2001/11/14 12:57:04 maranget Exp $"
+let header = "$Id: index.ml,v 1.44 2006-04-25 08:23:59 maranget Exp $"
 open Misc
 open Parse_opts
 open Entry
@@ -74,7 +74,7 @@ let read_index_file name file =
         with Entry.NoGood ->
           Misc.warning
           ("Bad index arg syntax in file: "^name^
-           ", index entry is ``"^arg1^"''") ;
+           ", index entry is '"^arg1^"'") ;
           Bad in
       Table.emit r entry ;
       do_rec ()
@@ -122,7 +122,7 @@ let do_treat tag arg refvalue anchor =
       with
       | Entry.NoGood ->
           idx.onebad <- true ;
-          Misc.warning ("Bad index syntax: ``"^arg^"''") ;
+          Misc.warning ("Bad index syntax: '"^arg^"'") ;
           Bad in
     Table.emit from_doc entry ;
     lbl
@@ -245,7 +245,7 @@ let start_change s1 s2 = match s1,s2 with
 | _,"" -> false
 | _,_  -> Char.uppercase s1.[0] <> Char.uppercase s2.[0]
 
-let print_entry out tag entries bk k xs  =
+let print_entry out _ entries bk k xs  =
   let rp,rt = common bk k in
   close_prev out rp ;
   if fst rp = [] then
@@ -372,7 +372,7 @@ let finalize check =
   if check then begin
     let top_changed = ref false in
     Hashtbl.iter
-      (fun tag idx ->
+      (fun _ idx ->
         let entries = Table.trim idx.from_doc in
         let changed =
           match idx.from_file with
