@@ -10,7 +10,7 @@
 (***********************************************************************)
 
 
-let header = "$Id: info.ml,v 1.34 2005/03/04 15:56:35 maranget Exp $"
+let header = "$Id: info.ml,v 1.41 2006-10-10 11:02:04 maranget Exp $"
 
 open Misc
 open Text
@@ -20,13 +20,9 @@ exception Error of string
 type block = Text.block
 
 
-let iso =Text.iso;;
-let iso_string =Text.iso_string;;
 let set_out=Text.set_out;;
 let stop = Text.stop;;
 let restart = Text.restart;;
-let get_last_closed=Text.get_last_closed;;
-let set_last_closed=Text.set_last_closed;;
 let is_empty=Text.is_empty;;
 
 let get_fontsize=Text.get_fontsize;;
@@ -34,13 +30,16 @@ let nostyle=Text.nostyle;;
 let clearstyle=Text.clearstyle;;
 let open_mod=open_mod;;
 let erase_mods=Text.erase_mods;;
+let forget_par = Text.forget_par;;
+let open_par = Text.open_par;;
+let close_par = Text.close_par;;
 let par=Text.par;;
-let forget_par =Text.forget_par;;
 
 
 let open_block =Text.open_block;;
 let close_block =Text.close_block;;
 let force_block =Text.force_block;;
+let close_flow =Text.close_flow;;
 let insert_block =Text.insert_block;;
 let insert_attr =Text.insert_attr;;
 
@@ -55,12 +54,11 @@ let erase_display =Text.erase_display
 and standard_sup_sub = Text.standard_sup_sub
 and limit_sup_sub = Text.limit_sup_sub
 and int_sup_sub = Text.int_sup_sub
+and addvsize = Text.addvsize
 and over = Text.over
-and over_align = Text.over_align
 and left = Text.left
 and right = Text.right
 ;;
-let box_around_display = Text.box_around_display;;
 let set_dcount =Text.set_dcount;;
 let item = Text.item;;
 let nitem = Text.nitem;;
@@ -71,9 +69,9 @@ let open_aftergroup =Text.open_aftergroup;;
 let close_group =Text.close_group;;
 
 let put s = Text.put s
-;;
+and put_char c = Text.put_char c
+and put_unicode i = Text.put_unicode i
 
-let put_char c = Text.put_char c;;
 
 let flush_out =Text.flush_out;;
 let skip_line =Text.skip_line;;
