@@ -14,6 +14,7 @@ module type S =
   sig
     (* external entry points *)
     val no_prelude : unit -> unit
+    val translate_put_unicode : char -> unit
     val main : Lexing.lexbuf -> unit
     val expand_command : string -> Lexing.lexbuf -> unit
     val expand_command_no_skip : string -> Lexing.lexbuf -> unit
@@ -32,12 +33,14 @@ module type S =
     val top_close_block : string -> unit
     val check_alltt_skip : Lexing.lexbuf -> unit
     val skip_pop : Lexing.lexbuf -> unit
-(* ``def'' functions for initialisation only *)
+(* 'def' functions for initialisation only *)
     val def_code : string -> (Lexing.lexbuf -> unit) -> unit
     val def_name_code : string -> (string -> Lexing.lexbuf -> unit) -> unit
     val def_fun : string -> (string -> string) -> unit
+(* various calls of main scanner, should tidy that a bit *)
     val get_this_main : string -> string
     val get_this_arg_mbox : string arg -> string
+    val get_prim_onarg : string Lexstate.arg -> string
     val check_this_main : string -> bool
     val get_prim : string -> string
     val get_prim_arg : Lexing.lexbuf -> string
