@@ -7,14 +7,14 @@
 (*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
-(*  $Id: lexeme.mli,v 1.5 2005/02/25 17:49:18 maranget Exp $           *)
+(*  $Id: lexeme.mli,v 1.6 2005-06-24 08:32:21 maranget Exp $           *)
 (***********************************************************************)
 type tag =
   | TT |I |B |BIG |SMALL
   | STRIKE | S |U |FONT
   | EM |STRONG |DFN |CODE |SAMP
   | KBD |VAR |CITE |ABBR |ACRONYM 
-  | Q |SUB |SUP | A | SCRIPT | SPAN
+  | Q |SUB |SUP | A | SCRIPT | SPAN | STYLE
 
 type atag =
   | SIZE of string | COLOR of string | FACE of string
