@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: lexstate.ml,v 1.65 2005/03/08 15:15:03 maranget Exp $"
+let header = "$Id: lexstate.ml,v 1.68 2006-07-20 12:52:00 maranget Exp $"
 
 open Misc
 open Lexing
@@ -172,8 +172,6 @@ type env = string array ref
 type closenv = string array t
 
 
-
-
 (* catcodes *)
 
 let plain_of_char = function
@@ -190,11 +188,12 @@ let plain_of_char = function
   | '\'' -> 10
   | '`' -> 11
   | '-' -> 12
+  | '"' -> 13
   | c   ->
       raise
         (Fatal ("Internal catcode table error: '"^String.make 1 c^"'"))
 
-and plain = Array.create 13 true
+and plain = Array.create 14 true
 
 let is_plain c = plain.(plain_of_char c)
 and set_plain c = plain.(plain_of_char c) <- true
@@ -331,6 +330,17 @@ and end_normal () =
   restore_lexstate ()
 ;;
 
+let full_peek_char lexbuf =
+  let rec full_peek lexbuf =
+    try
+      Save.peek_next_char lexbuf
+    with Not_found ->
+      if Stack.empty stack_lexbuf then
+        raise Not_found
+      else
+        full_peek (previous_lexbuf ()) in
+  full_peek lexbuf
+        
 let full_save_arg eoferror mkarg parg lexfun lexbuf =
   let rec save_rec lexbuf =
     try
@@ -357,7 +367,7 @@ let full_save_arg eoferror mkarg parg lexfun lexbuf =
     let r = save_rec lexbuf in
     restore_lexstate () ;
     if !verbose > 2 then
-      prerr_endline ("Arg parsed: ``"^parg r^"''") ;
+      prerr_endline ("Arg parsed: '"^parg r^"'") ;
     r
   with
   | (Save.Error _ | Error _) as e ->
@@ -371,6 +381,48 @@ let full_save_arg eoferror mkarg parg lexfun lexbuf =
       raise e
 ;;
 
+let full_save_arg_limits eoferror parg lexfun lexbuf =
+  let rec save_rec opt lexbuf =
+    try
+      lexfun opt lexbuf
+    with Save.LimitEof r -> begin
+        if Stack.empty stack_lexbuf then
+          match r with
+          | None -> eoferror () 
+          | _ -> r
+        else begin
+          let lexbuf = previous_lexbuf () in
+          if !verbose > 1 then begin
+            prerr_endline "popping stack_lexbuf in full_save_arg_limits";
+            pretty_lexbuf lexbuf ;
+            prerr_args ()
+          end;
+          save_rec r lexbuf
+        end
+    end in
+
+  let start_pos = Location.get_pos () in
+  try 
+    Save.seen_par := false ;
+    save_lexstate () ;
+    let r = save_rec None lexbuf in
+    restore_lexstate () ;
+    if !verbose > 2 then
+      prerr_endline ("Arg parsed: '"^parg r^"'") ;
+    r
+  with
+  | (Save.Error _ | Error _) as e ->
+      restore_lexstate () ;
+      Save.seen_par := false ;
+      Location.print_this_pos start_pos ;
+      prerr_endline "Parsing of argument failed" ;
+      raise e
+  | e ->
+      restore_lexstate () ;
+      raise e
+;;
+
+
 type ok = No of string | Yes of string
 ;;
 
@@ -404,8 +456,6 @@ type sup_sub = {
   sub : string arg ;
 } 
 
-let mklimits x _ = x
-
 let plimits = function
   | Some Limits ->    "\\limits"
   | Some NoLimits ->  "\\nolimits"
@@ -419,7 +469,8 @@ let save_limits lexbuf =
   let rec do_rec res =
     try
       let r =
-        full_save_arg eof_over mklimits plimits Save.get_limits lexbuf in
+        full_save_arg_limits
+          eof_over plimits Save.get_limits lexbuf in
       match r with
       | None -> res
       | Some _ -> do_rec r
@@ -526,7 +577,8 @@ let rec parse_args_norm pat lexbuf = match pat with
 
 
 let skip_csname lexbuf =
-  let _ = Save.csname lexbuf (fun x -> x) in ()
+  let _ = Save.csname (fun x -> x) (fun x -> x) lexbuf in
+  ()
 
 
 let skip_opt lexbuf =
