@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: mathML.ml,v 1.21 2004/11/26 13:13:05 maranget Exp $" 
+let header = "$Id: mathML.ml,v 1.26 2006-04-13 16:55:56 maranget Exp $" 
 
 
 open Misc
@@ -35,8 +35,8 @@ let begin_item_display f is_freeze =
 
 and end_item_display () =
   let f,is_freeze = pop_freeze () in
-  let _ = close_flow_loc INTERN in
-  if close_flow_loc (OTHER "mrow") then
+  let _ = close_flow_loc check_empty INTERN in
+  if close_flow_loc check_empty (OTHER "mrow") then
     flags.ncols <- flags.ncols + 1;
   if !verbose > 2 then begin
     Printf.fprintf stderr "end_item_display: ncols=%d stck: " flags.ncols;
@@ -125,7 +125,7 @@ let open_display_varg _ = open_display ()
   
 let do_item_display _force =
   if !verbose > 2 then begin
-    prerr_endline ("Item Display ncols="^string_of_int flags.ncols^" table_inside="^sbool flags.table_inside)
+    prerr_endline ("Item Display in mathML ncols="^string_of_int flags.ncols^" table_inside="^sbool flags.table_inside)
   end ;
   let f,is_freeze = pop_freeze () in
   if ((*force && *)not flags.empty) || flags.table_inside then
@@ -329,10 +329,9 @@ let put s =
   if not s_blank then begin
     let s_op = is_op s
     and s_number = is_number s in
-    let save_last_closed = flags.last_closed in
     if is_open_delim s then open_delim ();
     let s_text = if is_close_delim s then is_close () else false in
-    if s_op || s_number then force_item_display ();
+    if (s_op || s_number) && !Lexstate.display then force_item_display ();
     do_pending () ;
     flags.empty <- false;
     flags.blank <- s_blank && flags.blank ;
@@ -347,13 +346,11 @@ let put s =
     end else begin
       do_put s
     end;
-    if s_blank then flags.last_closed <- save_last_closed;
     if is_close_delim s then close_delim ()
   end
 ;;
 
 let put_char c =
-  let save_last_closed = flags.last_closed in
   let c_blank = is_blank c in
   if c <> ' ' then begin
     let s = String.make 1 c in
@@ -361,7 +358,7 @@ let put_char c =
     let c_digit = is_digit c in
     if is_open_delim s then open_delim ();
     let c_text = if is_close_delim s then is_close () else false in
-    if c_op || c_digit then force_item_display ();
+    if (c_op || c_digit) && !Lexstate.display then force_item_display ();
     do_pending () ;
     flags.empty <- false;
     flags.blank <- c_blank && flags.blank ;
@@ -374,7 +371,6 @@ let put_char c =
     end else begin
       do_put_char c;
     end;
-    if c_blank then flags.last_closed <- save_last_closed;
     if is_close_delim s then close_delim ();
   end
 ;;
@@ -383,7 +379,7 @@ let put_in_math s =
   if flags.in_pre && !pedantic then
     put s
   else begin
-    force_item_display ();
+    if !Lexstate.display then force_item_display ();
     do_pending () ;
     do_put "<mi> ";
     do_put s;
@@ -506,31 +502,27 @@ let int_sup_sub _something _vsize scanner what sup sub display =
 ;;
 
 
-let over display lexbuf =
- if display then begin
-    force_item_display ();
-    let _mods = insert_vdisplay
-        (fun () ->
-          open_block (OTHER "mfrac") "";
-	  open_display ()) in
-    force_item_display ();
-    flags.ncols <- flags.ncols +1;
-    close_display () ;
-    open_display () ;
-    freeze
+let over _lexbuf =
+  force_item_display ();
+  let _mods = insert_vdisplay
       (fun () ->
-	force_item_display ();
-	flags.ncols <- flags.ncols +1;
-        close_display () ;
-        close_block (OTHER "mfrac"))
-  end else begin
-    put "/"
-  end
+        open_block (OTHER "mfrac") "";
+	open_display ()) in
+  force_item_display ();
+  flags.ncols <- flags.ncols +1;
+  close_display () ;
+  open_display () ;
+  freeze
+    (fun () ->
+      force_item_display ();
+      flags.ncols <- flags.ncols +1;
+      close_display () ;
+      close_block (OTHER "mfrac"))
 ;;
 
 let box_around_display _scanner _arg = ();;
 
-let over_align _align1 _align2 display lexbuf = over display lexbuf
+let over_align _align1 _align2 display lexbuf = over lexbuf
 ;;
 
 let tr = function
@@ -541,7 +533,7 @@ let tr = function
 | s   -> s
 ;;
 
-let left delim k = 
+let left delim _ k = 
   force_item_display ();
   open_display ();
   if delim <>"." then put ("<mo> "^ tr delim^" </mo>");
@@ -556,10 +548,10 @@ let left delim k =
       close_display ();)
 ;;
 
-let right delim =
-  force_item_display ();
+let right delim _ =
+  if !Lexstate.display then force_item_display ();
   if delim <> "." then put ("<mo> "^tr delim^" </mo>");
-  force_item_display ();
+  if !Lexstate.display then force_item_display ();
   let f,is_freeze = pop_freeze () in
   if not is_freeze then begin
     warning "Right delimitor alone";
