@@ -9,8 +9,6 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: misc.ml,v 1.22 2001/10/19 18:35:58 maranget Exp $" 
-
 exception Fatal of string
 exception NoSupport of string
 exception Purposly of string
@@ -19,18 +17,18 @@ exception UserError of string
 exception EndInput
 exception EndDocument
 exception Close of string
+exception CannotPut
 exception EndOfLispComment of int (* QNC *)
 
 let verbose = ref 0
 and readverb = ref 0
+and displayverb = ref false
 
 let silent = ref false
 
 let column_to_command s = "\\@"^s^"@"
 
 
-let hot_start () = ()
-
 let warning s =
   if not !silent || !verbose > 0 then begin
     Location.print_pos () ;
@@ -114,3 +112,10 @@ let get_image_opt () = match !image_opt with
 
 
 let dump_index = ref false
+
+type saved = string option
+
+let checkpoint () = !image_opt
+
+and hot_start so = image_opt := so
+
