@@ -9,19 +9,27 @@
 (*                                                                     *)
 (***********************************************************************)
 
-exception Fatal of string
-exception NoSupport of string
-exception Purposly of string
-exception ScanError of string
-exception UserError of string
-exception EndInput
-exception EndDocument
-exception Close of string
+(***************)
+(* Some errors *)
+(***************)
+exception Fatal of string      (* A bug *)
+exception NoSupport of string  (* A feature *)
+exception Purposly of string   (* Explicit failure: \@heveafail command *)
+exception ScanError of string  (* Error while reading input *)
+exception UserError of string  (* Users should correct their code *)
+exception Close of string      (* Specific user error: env nesting *)
+
+(*******************)
+(* Some non-errors *)
+(*******************)
+exception EndInput    (* Stop reading that file *)
+exception EndDocument (* Document is over *)
 exception EndOfLispComment of int (* QNC *)
+exception CannotPut   (* Cannot ouput Unicode char (text mode) *)
 
-val hot_start : unit -> unit
 val verbose : int ref
 val readverb : int ref
+val displayverb : bool ref
 val silent : bool ref
 val column_to_command : string -> string
 val warning : string -> unit
@@ -31,10 +39,10 @@ val fatal : string -> 'a
 val not_supported : string -> 'a
 
 (* Copying hash tables, not very nice at present *)
+
 val copy_hashtbl : (string, 'a) Hashtbl.t -> (string, 'a) Hashtbl.t -> unit
-val clone_hashtbl : (string, 'a) Hashtbl.t -> (string, 'a) Hashtbl.t
 val copy_int_hashtbl : (int, 'a) Hashtbl.t -> (int, 'a) Hashtbl.t -> unit
-val clone_int_hashtbl : (int, 'a) Hashtbl.t -> (int, 'a) Hashtbl.t
+
 
 val start_env : string -> string
 val end_env : string -> string
@@ -44,3 +52,7 @@ type limits = Limits | NoLimits | IntLimits
 val image_opt : string option ref
 val get_image_opt : unit -> string
 val dump_index : bool ref
+
+type saved
+val checkpoint : unit -> saved
+val hot_start : saved -> unit
