@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(*  $Id: package.ml,v 1.69 2005/03/08 15:15:03 maranget Exp $    *)
+(*  $Id: package.ml,v 1.96 2006-10-16 08:57:17 maranget Exp $    *)
 
 module type S = sig  end
 
@@ -26,13 +26,180 @@ open Stack
 open Scan
 ;;
 
-(* Various outworld information *)
-let def_print name s =
-  def_code name  (fun _ -> Dest.put (Dest.iso_string s))
+(*********************************************************)
+(* Accents are here, to make latexscan.mll a bit smaller *)
+(* Accent commands use a mapping from ascii to           *)
+(* unicode entities with accents                         *)   
+(*  - When no char with accent exists (html) or can      *)
+(*    be outputed (text), command                        *)
+(*    \text@accent{\cmd}{name}{arg} is scanned           *)
+(* where \cmd is accent command name                     *)
+(*       name is some internal name (eg \' -> accute)    *)
+(*       arg is the argument to \cmd                     *)
+(* See iso-sym.hva, for the definition of \text@accent   *)
+(*********************************************************)
+
+let put_empty empty =
+  if empty <> 0 then Dest.put_unicode empty
+  else raise OutUnicode.CannotTranslate 
+
+exception DiacriticFailed of string
+
+let do_def_diacritic verb name f empty = 
+  (fun lexbuf ->
+    Save.start_echo () ;
+    let arg0 = save_arg lexbuf in
+    let arg = get_prim_onarg arg0 in
+    try match String.length arg with
+    | 0 -> put_empty empty
+    | 1 ->
+        let c = arg.[0] in
+        if c = ' ' then put_empty empty
+        else Dest.put_unicode (f arg.[0])
+    | _ -> raise OutUnicode.CannotTranslate
+    with
+    | OutUnicode.CannotTranslate
+    | Misc.CannotPut ->	raise (DiacriticFailed arg))
+
+let def_diacritic name internal f empty =
+  def_code name
+    (fun lexbuf ->
+      try do_def_diacritic true name f empty lexbuf
+      with DiacriticFailed input ->
+	scan_this main
+	  ("\\text@accent{"^internal^"}{"^name^"}{"^input^"}"))
 ;;
 
-def_code "\\@lexbuf"
+def_diacritic "\\'"  "acute" OutUnicode.acute 0xB4 ;
+def_diacritic "\\`"  "grave" OutUnicode.grave 0x60 ;
+def_diacritic "\\^"  "circumflex" OutUnicode.circumflex 0x5E ;
+def_diacritic "\\\"" "diaeresis" OutUnicode.diaeresis 0xA8 ;
+def_diacritic "\\c"  "cedilla" OutUnicode.cedilla 0xB8 ;
+def_diacritic "\\~"  "tilde" OutUnicode.tilde 0x7E ;
+def_diacritic "\\="  "macron" OutUnicode.macron 0xAF ;
+def_diacritic "\\H"  "doubleacute" OutUnicode.doubleacute 0x2DD ;
+(* package fc, see later ? *)
+(* def_diacritic "\\G"  "doublegrave" OutUnicode.doublegrave 0x2F5 ; *)
+def_diacritic "\\u"  "breve" OutUnicode.breve 0x2D8 ;
+def_diacritic "\\."  "dotabove" OutUnicode.dotabove 0x2D9 ;
+def_diacritic "\\d"  "dotbelow" OutUnicode.dotbelow 0x2D4 ;
+def_diacritic "\\b"  "linebelow" OutUnicode.linebelow 0x5F ;
+def_diacritic "\\k"  "ogonek" OutUnicode.ogonek 0x2DB ;
+def_diacritic "\\r"  "ringabove" OutUnicode.ring  0x2DA ;
+def_diacritic "\\v"  "caron" OutUnicode.caron 0x2C7 ;
+def_diacritic "\\textcircled" "circled" OutUnicode.circled 0x25EF ;
+(* activated by amssymb *)
+def_diacritic "\\@doublestruck" "doublestruck" OutUnicode.doublestruck 0 ;
+()
+;;
+
+(*
+ Specialized version of \IfDisplay (plain.hva) for command names,
+ so as to avoid inserting \fi at end of scanned text
+*)
+def_code "\\DisplayChoose"
   (fun lexbuf ->
+    let cmd1 = get_csname lexbuf in
+    let cmd2 = get_csname lexbuf in
+    if !display then
+      expand_command cmd1 lexbuf
+    else
+      expand_command cmd2 lexbuf)
+;;
+
+(**************)
+(* Delimiters *)
+(**************)
+def_code "\\process@delim@one"
+  (fun lexbuf ->
+    let n = Get.get_int (save_arg lexbuf) in
+    let n = if n < 2 then 2 else n in
+    let mid = get_csname lexbuf in
+    for _i = 1 to n-1 do
+      scan_this main mid ; Dest.skip_line () ;
+    done ;
+    scan_this main mid)
+;;
+
+
+def_code "\\process@delim@top"
+  (fun lexbuf ->
+    let n = Get.get_int (save_arg lexbuf) in
+    let top = get_csname lexbuf in
+    let mid = get_csname lexbuf in
+    scan_this main top ;
+    if n > 1 then begin
+      Dest.skip_line () ; scan_this main mid ;
+      for _i = 1 to n-2 do
+        Dest.skip_line () ; scan_this main mid ;
+      done
+    end)
+;;
+
+def_code "\\process@delim@dow"
+  (fun lexbuf ->
+    let n = Get.get_int (save_arg lexbuf) in
+    let mid = get_csname lexbuf in
+    let dow = get_csname lexbuf in
+    for _i = 1 to n-1 do
+      scan_this main mid ; Dest.skip_line () ;
+    done ;
+    scan_this main dow)
+;;
+
+def_code "\\process@delim@three"
+  (fun lexbuf ->
+    let n = Get.get_int (save_arg lexbuf) in
+    let top = get_csname lexbuf in
+    let mid = get_csname lexbuf in
+    let dow = get_csname lexbuf in
+    scan_this main top ; scan_this main "\\@top@br" ;
+    for _i = 1 to n-2 do
+      scan_this main mid ; Dest.skip_line () ;
+    done ;
+    scan_this main dow)
+;;
+
+def_code "\\process@delim@four"
+  (fun lexbuf ->
+    let n = Get.get_int (save_arg lexbuf) in
+    let ext = get_csname lexbuf in
+    let top = get_csname lexbuf in
+    let mid = get_csname lexbuf in
+    let dow = get_csname lexbuf in
+    scan_this main top ; Dest.skip_line () ;
+    for _i = 1 to (n-3+1)/2 do
+      scan_this main ext ; Dest.skip_line () ;
+    done ;
+    scan_this main mid ;  Dest.skip_line () ;
+    for _i = 1 to (n-3+1)/2 do
+      scan_this main ext ; Dest.skip_line () ;
+    done ;
+    scan_this main dow)
+;;
+
+let int_sup_sub lexbuf =
+  let n = Get.get_int (save_arg lexbuf) in
+  if !display then begin
+    let {limits=limits ; sup=sup ; sub=sub} = save_sup_sub lexbuf in
+    Dest.int_sup_sub false n
+      (scan_this_arg main) (fun () -> ()) sup sub true
+  end
+;;
+
+def_code "\\int@sup@sub" int_sup_sub
+;;
+
+(* Direct ahustement of vsize *)
+def_code "\\@addvsize"
+  (fun lexbuf ->
+    let n =  Get.get_int (save_arg lexbuf) in
+    Dest.addvsize n)
+;;
+
+(* Various outworld information *)
+def_code "\\@lexbuf"
+  (fun _ ->
     prerr_endline ("LEXBUF: "^string_of_int (Stack.length stack_lexbuf)))
 ;;
 
@@ -40,6 +207,16 @@ def_code "\\@macros"
   (fun _ -> Latexmacros.pretty_table ())
 ;;
 
+let translate_put s =
+  for k=0 to String.length s-1 do
+    Scan.translate_put_unicode s.[k]
+  done
+;;
+
+let def_print name s =
+  def_code name (fun _ -> translate_put s)
+;;
+
 def_print "\\@basein" Parse_opts.base_in ;
 def_print "\\jobname" Parse_opts.base_out ;
 def_print "\\@heveacomline"
@@ -50,15 +227,60 @@ def_print "\\@heveaversion" Version.version ;
 def_print "\\@hevealibdir" Mylib.libdir
 ;;
 
-(* Unicode entities given in hexa *)
-def_code "\\@unicode"
- (fun lexbuf ->
-   let arg = get_prim_arg lexbuf in
-   Scanf.sscanf arg "%x" (fun x -> Dest.put ("&#"^string_of_int x^";")))
+def_code "\\@heveaverbose"
+  (fun lexbuf ->    
+    let lvl = Get.get_int (save_arg lexbuf) in
+    Misc.verbose := lvl)
 ;;
 
-     
-(* ``Token'' registers *)
+(* External style-sheet *)
+def_code "\\hva@dump@css"
+  (fun _ ->
+     let name = match Parse_opts.base_out with
+     | "" ->
+         let r = "out.css" in
+         warning ("Outputing style sheet to default file: "^r) ;
+         r
+     | base -> base ^ ".css" in
+     begin try
+       let stys =
+         Dest.to_string 
+           (fun () -> scan_this main "\\hevea@css") in
+       let chan = open_out name in
+       output_string chan stys ;
+       close_out chan
+     with
+     | Sys_error msg ->
+         warning ("Trouble while outputing style sheet: "^msg)
+     end ;
+     scan_this main (Printf.sprintf "\\@getprintnostyle{%s}" name))
+;;
+
+
+(* Stacks of command definitions *)
+
+def_code "\\@hva@newstack"
+  (fun lexbuf ->
+     let name = get_prim_arg lexbuf in
+     let stack = Stack.create name in
+     def_code
+       ("\\@push"^name)
+       (fun lexbuf ->
+         let cmd = Scan.get_csname lexbuf in
+         let def = Latexmacros.find cmd in
+         Stack.push stack def) ;
+     def_code
+       ("\\@pop"^name)
+       (fun lexbuf ->
+         let cmd = Scan.get_csname lexbuf in
+         try
+           let pat,body = Stack.pop stack in
+           Latexmacros.def cmd pat body
+         with Fatal _ ->
+           warning (Printf.sprintf "Pop empty stack '%s'" name)))
+;;
+
+(* 'Token' registers *)
 def_code "\\newtokens"
   (fun lexbuf ->
     let toks = Scan.get_csname lexbuf in
@@ -67,13 +289,17 @@ def_code "\\newtokens"
     Latexmacros.def toks zero_pat (Toks []))
 ;;
 
+let get_tokens toks = match Latexmacros.find_fail toks with
+| _,Toks r-> r
+| _ -> raise Failed
+;;
+
 def_code "\\resettokens"
   (fun lexbuf ->
-    let toks = Scan.get_csname lexbuf in
-    begin try match Latexmacros.find_fail toks with
-    | _,Toks _ ->
-        Latexmacros.def toks zero_pat (Toks [])
-    | _ -> raise Failed
+    let toks = Scan.get_csname lexbuf in    
+    begin try
+      ignore (get_tokens toks) ;
+      Latexmacros.def toks zero_pat (Toks [])
     with Failed ->
       Misc.warning ("\\resettokens for "^toks^" failed")
     end)
@@ -83,27 +309,48 @@ def_code "\\addtokens"
   (fun lexbuf ->
     let toks = Scan.get_csname lexbuf in
     let arg = Subst.subst_arg lexbuf in
-    begin try match Latexmacros.find_fail toks with
-    | _,Toks l ->
-        Latexmacros.def toks zero_pat (Toks (arg::l))
-    | _ -> raise Failed
+    begin try
+      let l = get_tokens toks in
+      Latexmacros.def toks zero_pat (Toks (arg::l))
     with Failed ->
       Misc.warning ("\\addtokens for "^toks^" failed")
     end)
 ;;
+
 def_code "\\addrevtokens"
   (fun lexbuf ->
     let toks = Scan.get_csname lexbuf in
     let arg = Subst.subst_arg lexbuf in
-    begin try match Latexmacros.find_fail toks with
-    | _,Toks l ->
-        Latexmacros.def toks zero_pat (Toks (l@[arg]))
-    | _ -> raise Failed
+    begin try 
+      let l = get_tokens toks in
+      Latexmacros.def toks zero_pat (Toks (l@[arg]))
     with Failed ->
       Misc.warning ("\\addtokens for "^toks^" failed")
     end)
 ;;
 
+def_code "\\appendtokens"
+ (fun lexbuf ->
+   let toks1 = Scan.get_csname lexbuf in
+   let toks2 = Scan.get_csname lexbuf in
+   begin try
+     let l1 = get_tokens toks1
+     and l2 = get_tokens toks2 in
+      Latexmacros.def toks1 zero_pat (Toks (l2@l1))
+   with Failed ->
+     Misc.warning ("\\addtokens for "^toks1^" and "^toks2^" failed")
+   end)
+;;
+
+def_code "\\typemacro"
+  (fun lexbuf ->
+    let name = Scan.get_csname lexbuf in
+    let pat,body = Latexmacros.find name in
+    Latexmacros.pretty_macro pat body)
+;;
+
+(* See also the lrtokens env in latexscan.mll *)
+
 let call_subst lexbuf =
   let csname = get_csname lexbuf in
   let arg = subst_arg lexbuf in
@@ -268,6 +515,7 @@ register_init "index"
         let tag = get_prim_opt "default" lexbuf in
         let name = get_prim_arg lexbuf in
         Index.changename tag name) ;
+
     let new_index lexbuf =
       let tag = get_prim_arg lexbuf in
       let sufin = get_prim_arg lexbuf in
@@ -379,6 +627,18 @@ register_init "colortbl"
           Dest.insert_attr "TR" ("bgcolor=\""^color_to_string htmlval^"\"")))
 ;;
 
+(* xspace package *)
+register_init "xspace"
+  (fun () ->
+    def_code "\\xspace"
+    (fun lexbuf ->
+      try match Lexstate.full_peek_char lexbuf with
+      | '{'|'}'|'~'|'.'|'!'|','|':'|'?'|'/'|'\''|')'|'-'
+      | ' '|'\t'|'\n' -> ()
+      |  _ -> Dest.put_char ' '
+      with Not_found ->
+        warning "\\xspace could not reach next char"))
+;;
 
 (* sword package *)
 register_init "sword"
@@ -449,9 +709,7 @@ register_init "graphics"
 (* url package *)
 let verb_arg lexbuf =
   let {arg=url} = save_verbatim lexbuf in
-  for i = 0 to String.length url - 1 do
-    Dest.put (Dest.iso url.[i])
-  done
+  translate_put url
 ;;
 
 def_code "\\@verbarg" verb_arg ;
@@ -621,18 +879,9 @@ register_init "keyval"
   )
 ;;
 
-register_init "amsmath"
-  (fun () ->
-    def_code "\\numberwithin"
-      (fun lexbuf ->
-        let name = get_prim_arg lexbuf in
-        let within = get_prim_arg  lexbuf in
-        Counter.number_within name within)
-  )
-;;
-
 register_init "xypic"
-  (fun () -> def_code "\\@xyarg" 
+  (fun () ->
+    def_code "\\@xyarg" 
       (fun lexbuf ->
         Save.start_echo () ;
         let _ = Lexstate.save_xy_arg lexbuf in
@@ -664,6 +913,101 @@ register_init "natbib"
     ())
 ;;            
 
+let gput lexbuf c =
+  Save.gobble_one_char lexbuf ;
+  Dest.put_char c
+
+and gput_unicode lexbuf u =
+  Save.gobble_one_char lexbuf ;
+  Dest.put_unicode u
+;;
+
+let gscan lexbuf cmd =
+  Save.gobble_one_char lexbuf ;
+  Scan.expand_command_no_skip cmd lexbuf
+;;
+
+
+register_init "german"
+  (fun () ->
+      def_code "\\@german@dquote"
+        (fun lexbuf ->
+          if effective !alltt then
+            Dest.put_char '"'
+          else try
+            let c = Save.peek_next_char lexbuf  in
+            match c with
+            | 'a' | 'A' | 'e' | 'E' | 'i' | 'I' | 'o' | 'O' | 'u'| 'U' ->
+                begin try gput_unicode lexbuf (OutUnicode.diaeresis c)
+                with OutUnicode.CannotTranslate -> assert false end
+            | 's'|'z' -> gput_unicode lexbuf 0xDF
+            | 'c'|'f'|'l'|'m'|'p'|'r'|'t' as c ->
+                gput lexbuf c (* for "ck and "ff etc. *)
+            | 'S' ->
+                Save.gobble_one_char lexbuf ;
+                Dest.put "SS"
+            | 'Z' ->
+                Save.gobble_one_char lexbuf ;
+                Dest.put "SZ"
+            | '|' | '-'| '"' -> Save.gobble_one_char lexbuf
+            | '~'|'=' -> gput lexbuf '-'
+            | '`' -> gscan lexbuf "\\glqq"
+            | '\'' -> gscan lexbuf "\\grqq"
+            | '<' -> gscan lexbuf "\\flqq"
+            | '>' -> gscan lexbuf "\\frqq"
+            | _ -> Dest.put_char '"'
+          with
+          | Not_found -> Dest.put_char '"'))
+;;
+
+(***************)
+(* Translators *)
+(***************)
+
+def_code "\\@set@translators"
+    (fun lexbuf ->
+      let name = get_prim_arg lexbuf in
+      OutUnicode.set_translators name)
+;;
+
+def_code "\\@set@out@translator"
+  (fun lexbuf ->
+    let name = get_prim_arg lexbuf in
+    OutUnicode.set_output_translator name)
+;;
+
+def_code "\\@set@in@translator"
+  (fun lexbuf ->
+    let name = get_prim_arg lexbuf in
+    OutUnicode.set_input_translator name)
+;;
+
+(********************************)
+(* Counters, change reset lists *)
+(********************************)
+
+def_code "\\@addtoreset"
+  (fun lexbuf ->
+    let name = get_prim_arg lexbuf in
+    let within = get_prim_arg lexbuf in
+    Counter.addtoreset name within)
+;;
+
+register_init "chngcntr"
+  (fun () ->
+    def_code "\\@removefromreset"
+      (fun lexbuf ->
+	let name = get_prim_arg lexbuf in
+	let within = get_prim_arg lexbuf in
+	Counter.removefromreset name within) ;
+    ())
+;;
+
+
+(*************************************)
+(* Extra font changes, from abisheck *)
+(*************************************)
+
 let get_elements str = 
     let len = String.length str in
     let rec all_elements l = match l with
@@ -761,125 +1105,4 @@ def_code "\\@mathtt"
 ;;
 
 
-(*
-def_code "\\xrightarrow"
-  (fun lexbuf ->
-    let optarg = save_opt "" lexbuf in
-    let arg = save_arg lexbuf in
-    Dest.open_block "TABLE" "border=0 cellpadding=0 cellspacing=0";
-    Dest.open_block "SMALL" "" ;
-    Dest.open_block "TR" "" ;
-    Dest.open_block "TD" "" ;
-    Dest.put "&nbsp;&nbsp; " ;
-    Dest.close_block "TD" ;
-    Dest.open_block "TD" "align=center valign=bottom" ;
-    scan_this_arg Scan.main arg;
-    Dest.close_block "TD" ;
-    Dest.open_block "TD" "color=black" ;
-    Dest.put "&nbsp;" ;
-    Dest.close_block "TD" ;
-    Dest.close_block "TR" ;
-    Dest.close_block "SMALL";
-
-    Dest.open_block "TR" "" ;
-    Dest.open_block "TD" "ALIGN=right" ;
-    Dest.put ("<TABLE border=0 cellspacing=0 cellpadding=1 bgcolor=black "^
-	      "width=\"100%\"><TR><TD></TD></TR></TABLE>") ;
-    Dest.close_block "TD" ;
-    Dest.open_block "TD" "align=center" ;
-    Dest.put ("<TABLE border=0 cellspacing=0 cellpadding=1 bgcolor=black "^
-	      "width=\"100%\"><TR><TD></TD></TR></TABLE>") ;
-    Dest.close_block "TD" ;
-    Dest.open_block "TD" "ALIGN=left" ;
-    Dest.put "&gt" ;
-    Dest.close_block "TD" ; 
-    Dest.close_block "TR" ;
-
-    Dest.open_block "SMALL" "" ;
-    Dest.open_block "TR" "" ;
-    Dest.open_block "TD" "" ;
-    Dest.put "&nbsp;" ;
-    Dest.close_block "TD" ;
-    Dest.open_block "TD" "align=center valign=top" ;
-    scan_this_arg Scan.main optarg;
-    Dest.close_block "TD" ;
-    Dest.open_block "TD" "" ;
-    Dest.put "&nbsp;" ;
-    Dest.close_block "TD" ;
-    Dest.close_block "TR" ;
-    Dest.close_block "SMALL" ;
-    Dest.close_block "TABLE")
-;;
-
-
-def_code "\\xleftarrow"
-  (fun lexbuf ->
-    let optarg = save_opt "" lexbuf in
-    let arg = save_arg lexbuf in
-    Dest.open_block "TABLE" "border=0 cellpadding=0 cellspacing=0";
-    Dest.open_block "SMALL" "" ;
-    
-    Dest.open_block "TR" "" ;
-    Dest.open_block "TD" "align=center valign=bottom" ;
-    scan_this_arg Scan.main arg;
-    Dest.close_block "TD" ;
-    Dest.close_block "TR" ;
-
-    Dest.open_block "TR" "" ;
-    Dest.open_block "TD" "align=center style=\"font-size:7pt;\"" ;
-    Dest.put "&larr;&mdash;&mdash;&mdash;&mdash;" ;
-    Dest.close_block "TD" ; 
-    Dest.close_block "TR" ;
-
-    Dest.open_block "TR" "" ;
-    Dest.open_block "TD" "align=center valign=top" ;
-    scan_this_arg Scan.main optarg;
-    Dest.close_block "TD" ;
-    Dest.close_block "TR" ;
-    Dest.close_block "SMALL" ;
-    Dest.close_block "TABLE")
-;;
-*)
-(*
-def_code "\\cfrac"
-  (fun lexbuf ->
-    let optarg = save_opt "" lexbuf in
-    let arg_up = save_arg lexbuf in
-    let arg_down = save_arg lexbuf in
-    let opt_str = optarg.arg in
-    let align = match opt_str with
-      "l" -> "align=left"
-    | "r" -> "align=right"
-    | "" ->  "align=center"  
-    | _ -> "align=center" (* WARNING : need to give appropriate warning *) in
-    Dest.open_block "TABLE" "border=0 cellpadding=0 cellspacing=0";
-        
-    Dest.open_block "TR" "" ;
-    Dest.open_block "TD" (align^" valign=bottom") ;
-    scan_this_arg Scan.main arg_up;
-    Dest.close_block "TD" ;
-    Dest.close_block "TR" ;
-
-    Dest.open_block "TR" "" ;
-    Dest.open_block "TD" "bgcolor=black" ;
-    Dest.open_block "TABLE" 
-      "border=0 width=\"100%\" cellspacing=0 cellpadding=1" ;
-    Dest.open_block "TR" "" ;
-    Dest.open_block "TD" "";
-    Dest.close_block "TD" ; 
-    Dest.close_block "TR" ;
-    Dest.close_block "TABLE" ;
-    Dest.close_block "TD" ; 
-    Dest.close_block "TR" ;
-
-    Dest.open_block "TR" "" ;
-    Dest.open_block "TD" (align^" valign=top") ;
-    scan_this_arg Scan.main arg_down;
-    Dest.close_block "TD" ;
-    Dest.close_block "TR" ;
-    Dest.close_block "TABLE")
-;;
-*)
-
-
- end
+end
