@@ -11,17 +11,15 @@
 
 open Misc
 
-let header = "$Id: parse_opts.ml,v 1.32 2005/02/14 09:28:53 maranget Exp $" 
+let header = "$Id: parse_opts.ml,v 1.36 2006-09-29 13:53:59 maranget Exp $" 
 
 type input = File of string | Prog of string
 
 let files = ref []
 ;;
 
-let add_input s =
-  files := File s :: !files
-and add_program s =
-  files := Prog s :: !files
+let add_input s = files := File s :: !files
+and add_program s = files := Prog s :: !files
 ;;
 
 (* use this to create your warnings if you wish to *)
@@ -42,7 +40,6 @@ let moreentities = ref false
 type symbol_mode = SText | Symbol | Entity
 
 let symbol_mode = ref Entity
-and iso = ref true
 and pedantic = ref false
 and destination = ref Html
 and fixpoint = ref false
@@ -68,6 +65,8 @@ let path = ref []
 let outname = ref ""
 ;;
 
+let check_displayverb n = if n > 1 then displayverb := true
+
 let _ = Arg.parse
     [
   ("-version", Arg.Unit
@@ -76,8 +75,12 @@ let _ = Arg.parse
        print_endline ("library directory: "^Mylib.static_libdir) ;
        exit 0),
    "show hevea version and library directory") ;
-  ("-v", Arg.Unit (fun () -> readverb := !readverb + 1),
+  ("-v",
+   Arg.Unit
+     (fun () -> readverb := !readverb + 1 ; check_displayverb !readverb),
    "verbose flag, can be repeated to increase verbosity") ;
+  ("-dv", Arg.Unit (fun () -> displayverb := true),
+   "add borders to some block-level elements, so as to show hevea output structure") ;
   ("-s", Arg.Unit (fun () -> silent := true),
    "suppress warnings") ;
   ("-I", Arg.String (fun s -> path := s :: !path),
@@ -100,13 +103,11 @@ let _ = Arg.parse
    "Enable the output of some rare entities.") ;
   ("-entities", Arg.Unit (fun () -> symbol_mode := Entity),
    "Render symbols by using entities, this is the default") ;
-  ("-symbols", Arg.Unit (fun () -> symbol_mode := Symbol),
-   "Render symbols by using the symbol font, obsolete") ;
   ("-textsymbols", Arg.Unit (fun () -> symbol_mode := SText),
    "Render symbols by english text") ;
 
-  ("-noiso",Arg.Unit (fun () -> iso := false),
-   "use HTML entities in place of isolatin1 non-ascii characters") ;
+  ("-noiso",Arg.Unit (fun () -> Misc.warning "-noiso is deprecated, by default hevea output is ascii"),
+   "deprecated, does nothing") ;
   ("-pedantic",Arg.Unit (fun () -> pedantic := true),
    "be pedantic in interpreting HTML 4.0 transitional definition") ;
   ("-mathml",Arg.Unit (fun() -> mathml := true),
