@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: text.ml,v 1.65 2005/03/29 10:46:27 maranget Exp $"
+let header = "$Id: text.ml,v 1.79 2006-10-10 11:02:04 maranget Exp $"
 
 
 open Misc
@@ -23,105 +23,6 @@ open Length
 exception Error of string;;
 type block = string
 
-
-let r_quote = String.create 1
-;;
-
-let quote c =
-  (r_quote.[0] <- c ; r_quote)
-;;
-
-let r_translate = String.create 1
-;;
-
-let iso_translate = function
-| '' -> "!"
-| '' -> "cent"
-| '' -> "pound"
-| '' -> "curren"
-| '' -> "yen"
-| '' -> "I"
-| '' -> "paragraphe"
-| '' -> "trema"
-| '' -> "copyright"
-| '' -> "a"
-| '' -> "<<"
-| '' -> "not"
-| '' -> "-"
-| '' -> "registered"
-| '' -> "-"
-| '' -> "degre"
-| '' -> "plus ou moins"
-| '' -> "carre"
-| '' -> "cube"
-| '' -> "'"
-| '' -> "mu"
-| '' -> ""
-| '' -> "."
-| '' -> ""
-| '' -> "1"
-| '' -> "eme"
-| '' -> ">>"
-| '' -> "1/4"
-| '' -> "1/2"
-| '' -> "3/4"
-| '' -> "?"
-| '' -> "A"
-| '' -> "A"
-| '' -> "A"
-| '' -> "A"
-| '' -> "A"
-| '' -> "A"
-| '' -> "AE"
-| '' -> "C"
-| '' -> "E"
-| '' | '' | '' -> "E"
-| '' | '' | '' | '' -> "I"
-| '' -> "D"
-| '' -> "N"
-| '' | '' | '' | '' | '' -> "O"
-| '' -> "x"
-| '' -> "0"
-| '' | '' | '' | '' -> "U"
-| '' -> "Y"
-| '' -> "P"
-| '' -> "ss"
-| '' | '' | '' | '' | '' | '' -> "a"
-| '' -> "ae"
-| '' -> "c"
-| '' | '' | '' | '' -> "e"
-| '' | '' | '' | '' -> "i"
-| '' -> "o"
-| '' -> "n"
-| '' | '' | '' | '' | '' -> "o"
-| '' -> "/"
-| '' -> "o"
-| '' | '' | '' | '' -> "u"
-| '' -> "y"
-| '' -> "y"
-| '' -> "y"
-| c   -> (r_translate.[0] <- c ; r_translate)
-;;
-
-let iso c =
-  if !Parse_opts.iso || !Lexstate.raw_chars then
-    (r_translate.[0]<-c; r_translate)
-  else
-    iso_translate c
-;;
-
-let iso_buff = Out.create_buff ()
-
-let iso_string s =
-  if !Parse_opts.iso then begin
-    for i = 0 to String.length s - 1 do
-      Out.put iso_buff (iso_translate s.[i])
-    done ;
-    Out.to_string iso_buff
-  end else
-    s
-
-
 let failclose s = raise (Misc.Close s)
 ;;
 
@@ -167,6 +68,7 @@ let pblock () =
   match top out_stack with
   | Normal (s,_,_) -> s
   | _ -> ""
+
 and parg () =
   if empty out_stack then "" else
   match top out_stack with
@@ -705,6 +607,9 @@ let close_mods () =
   !cur_out.active <- []
 ;;
 
+let open_par () = ()
+and close_par () = false
+
 let par = function (*Nombre de lignes a sauter avant le prochain put*)
   | Some n ->
       begin
@@ -720,6 +625,8 @@ let par = function (*Nombre de lignes a sauter avant le prochain put*)
   | _ -> ()
 
 
+
+	
 let forget_par () = 
   let r = flags.pending_par in
   flags.pending_par <- None;
@@ -730,7 +637,7 @@ let flush_par n =
   flags.pending_par <- None;
   let p = n in
   do_put_char '\n' ;
-  for i=1 to p-1 do
+  for _i=1 to p-1 do
     do_put_char '\n'
   done;
   if !verbose >2 then
@@ -810,7 +717,7 @@ let try_open_block s args =
 	flags.in_align<-true;
 	flags.align <- Left;
 	flags.first_line<-0;
-	flags.x_start<- flags.x_start + 20 * flags.hsize / 100;
+	flags.x_start<- flags.x_start + 5 * flags.hsize / 100;
 	flags.hsize <- flags.x_end - flags.x_start+1;
       end
   | "QUOTATION" ->
@@ -819,7 +726,7 @@ let try_open_block s args =
 	flags.in_align<-true;
 	flags.align <- Left;
 	flags.first_line<-2;
-	flags.x_start<- flags.x_start + 20 * flags.hsize / 100;
+	flags.x_start<- flags.x_start + 5 * flags.hsize / 100;
 	flags.hsize <- flags.x_end - flags.x_start+1;
       end
   | "PRE" ->
@@ -844,7 +751,7 @@ let try_open_block s args =
 ;;
     
 let try_close_block s =
-  let (h,x,xs,xe,fl,lp) = pop stacks.s_x in
+  let (h,_,xs,xe,fl,_) = pop stacks.s_x in
   flags.hsize<-h; 
   flags.x_start<-xs;
   flags.x_end<-xe;
@@ -903,12 +810,12 @@ let open_block s args =
     prerr_endline ("<= open_block ``"^bloc^"''")
 ;;
 
-let force_block s content =  
+let force_block s _ =  
   if !verbose > 2 then
     prerr_endline ("   force_block ``"^s^"''");
   let old_out = !cur_out in
   try_close_block s;
-  let ps,pa,pout = pop_out out_stack in
+  let ps,_,pout = pop_out out_stack in
   if ps <>"DELAY" then begin
     cur_out:=pout;
     if ps = "AFTER" then begin
@@ -932,9 +839,15 @@ let close_block s =
     prerr_endline ("<= close_block ``"^bloc^"''");
 ;;
 
+(* Hum, wrong *)
+let close_flow s =
+  let active  = !cur_out.active in
+  close_block s ;
+  !cur_out.active <- active
+;;
 
 
-let insert_block tag arg =  match arg with
+let insert_block _ arg =  match arg with
 | "LEFT" -> flags.align <- Left
 | "CENTER" -> flags.align <- Center
 | "RIGHT" -> flags.align <- Right
@@ -1031,19 +944,24 @@ let close_group () =
 
 
 let put s =
-  if !verbose > 3 then
-    Printf.fprintf stderr "put: %s\n" s ;
+  if !verbose > 3 then Printf.fprintf stderr "put: %s\n" s ;
   do_pending ();
   do_put s
 ;;
 
 let put_char c =
-  if !verbose > 3 then
-    Printf.fprintf stderr "put_char: %c\n" c ;
+  if !verbose > 3 then Printf.fprintf stderr "put_char: %c\n" c ;
   do_pending ();
   do_put_char c
 ;;
 
+let put_unicode i =
+  if !verbose > 3 then Printf.fprintf stderr "put_unicode: %04X\n" i ;
+  do_pending ();
+  try do_put_char (OutUnicode.translate_out i)
+  with OutUnicode.CannotTranslate -> raise Misc.CannotPut
+;;
+
 let flush_out () =
   Out.flush !cur_out.out
 ;;
@@ -1054,7 +972,7 @@ let skip_line () =
   put_char '\n'
 ;;
 
-let loc_name s1 = ()
+let loc_name _ = ()
 ;;
 
 let open_chan chan =
@@ -1097,7 +1015,7 @@ let unskip () = do_unskip ()
 let put_separator () = put " "
 ;;
 
-let put_tag tag = ()
+let put_tag _ = ()
 ;;
 
 let put_nbsp () =  put " "
@@ -1220,7 +1138,7 @@ let multi = ref []
 and multi_stack = Stack.create "multi_stack";;
 
 
-let open_table border _ =
+let open_table _ _ =
   (* creation d'une table : on prepare les donnees : creation de l'environnement qvb, empilage du precedent. *)
   push table_stack !table;
   push row_stack !row;
@@ -1319,7 +1237,7 @@ let change_format format = match format with
 	  let n =
 	    try
 	      int_of_string s
-	    with (Failure fail) -> raise (Misc.Fatal ("open_cell, invalid vertical format :"^v));
+	    with (Failure _) -> raise (Misc.Fatal ("open_cell, invalid vertical format :"^v));
 	  in
 	  if n>100 || n<0 then raise (Misc.Fatal ("open_cell, invalid vertical format :"^v));
 	  Base n);
@@ -1438,7 +1356,7 @@ let close_cell content =
   end else if !cell.span = 0 then begin
     Table.emit !table.taille 0;
   end else begin
-    for i = 1 to !cell.span do
+    for _i = 1 to !cell.span do
       Table.emit !table.taille 0
     done;
     multi := (!table.col,!table.col + !cell.span -1,!cell.w) :: !multi;
@@ -1479,7 +1397,7 @@ let erase_row () =
   if !verbose > 2 then prerr_endline "erase_row" ;  
   !table.line <- !table.line -1
 
-and close_row erase =
+and close_row _ =
   if !verbose> 2  then
     Printf.eprintf "close_row tailles=%a, taille=%a\n"
       ptailles !table ptaille !table ;
@@ -1509,7 +1427,7 @@ let make_border s =
     !cell.post <- !cell.post ^ s
 ;;
 
-let make_inside s multi =
+let make_inside s _ =
   if !verbose>2 then prerr_endline ("Adding inside after column "^string_of_int !table.col^" :'"^s^"'");
   
   if (!table.col = -1) || not ( !table.in_cell) then begin
@@ -1528,7 +1446,7 @@ let make_inside s multi =
 ;;
 
 
-let make_hline w noborder =
+let make_hline _ _ =
   new_row();
   open_cell center_format 0 0;
   close_mods ();
@@ -1729,11 +1647,11 @@ let close_table () =
 (* Info *)
 
 
-let infomenu arg = ()
+let infomenu _ = ()
 ;;
 
-let infonode opt num arg = ()
-and infoextranode num arg text = ()
+let infonode _ _ _ = ()
+and infoextranode _ _ _ = ()
 ;;
 
 (* Divers *)
@@ -1749,7 +1667,7 @@ let is_blank s =
 let is_empty () =
   flags.in_table && (Out.is_empty !cur_out.out) && (flags.x= -1);;
 
-let image arg n = 
+let image arg _ = 
     if arg <> "" then begin
     put arg;
     put_char ' '
@@ -1921,9 +1839,8 @@ and close_maths display =
     prerr_endline "close_maths";
 ;;
 
-let box_around_display scanner arg = ();;
 
-let open_vdisplay display = 
+let open_vdisplay _ = 
   open_table (!verbose>1) "";
 
 and close_vdisplay () = 
@@ -2022,7 +1939,7 @@ and limit_sup_sub scanner what sup sub display =
   close_vdisplay ();
   item_display ();
 
-and int_sup_sub something vsize scanner what sup sub display =
+and int_sup_sub something _ scanner what sup sub display =
   if something then what ();
   item_display ();
   open_vdisplay display;
@@ -2068,28 +1985,22 @@ let insert_vdisplay open_fun =
   []
 ;;
 
+let addvsize x = flags.vsize <- flags.vsize + x
 
-
-let over display lexbuf =
-  if !verbose>1 then
-    prerr_endline "over";
-  if display then begin
-    let _=insert_vdisplay 
-	( fun () -> 
-	  begin
-	    open_vdisplay display;
-	    open_vdisplay_row "cm";
-	  end) in
+let over _ =
+  let _=insert_vdisplay 
+      ( fun () -> 
+	begin
+	  open_vdisplay display;
+	  open_vdisplay_row "cm"
+	end) in
+  close_vdisplay_row ();
+  make_hline 0 false ;
+  open_vdisplay_row "cm";
+  freeze (fun () ->
     close_vdisplay_row ();
-    make_hline 0 false;
-    open_vdisplay_row "cm";
-    freeze (fun () ->
-      close_vdisplay_row ();
-      close_vdisplay ();
-      close_display (););
-  end else begin
-    put "/";
-  end
+    close_vdisplay ();
+    close_display ();)
 
 let translate = function
   "<" -> "<"
@@ -2099,19 +2010,16 @@ let translate = function
 | s   -> s
 ;;
 
-let over_align align1 align2 display lexbuf = over display lexbuf
-;;
-
-let left delim k =
+let left delim _ k =
   item_display ();
   open_display "";
   close_cell_group ();
-  if delim<>"." then make_border (translate delim);
+  make_border (translate delim) ;
   k 3 ;
   open_cell_group ();
 ;;
 
-let right delim =
+let right delim _ =
   let vsize = 3 in
   if delim<>"." then make_border (translate delim);
   item_display ();
@@ -2119,7 +2027,3 @@ let right delim =
   vsize
 ;;
 
-(*
-  C'est fini, legamment
-*)
-
