@@ -9,44 +9,26 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: thread.ml,v 1.4 2000/05/22 12:19:14 maranget Exp $" 
+let header = "$Id: thread.ml,v 1.5 2006-01-30 08:56:26 maranget Exp $" 
+
 let uptable = Hashtbl.create 17
 and nexttable = Hashtbl.create 17
 and prevtable = Hashtbl.create 17
 ;;
 
-let setup file upname = Hashtbl.add uptable file (ref upname)
-and setprev file prevname = Hashtbl.add prevtable file (ref prevname)
-let setnext file nextname = Hashtbl.add nexttable file (ref nextname)
+let setup file upname = Hashtbl.add uptable file upname
+and setprev file prevname = Hashtbl.add prevtable file prevname
+let setnext file nextname = Hashtbl.add nexttable file nextname
 ;;
 
 let setprevnext prev now =
   if prev <> "" then begin
-    Hashtbl.add nexttable prev (ref now) ;
-    Hashtbl.add prevtable now (ref prev)
+    Hashtbl.add nexttable prev now ;
+    Hashtbl.add prevtable now prev
   end
 ;;
 
-let next name = !(Hashtbl.find nexttable name)
-and up   name = !(Hashtbl.find uptable name)
-and prev name = !(Hashtbl.find prevtable name)
+let next name = Hashtbl.find nexttable name
+and up   name = Hashtbl.find uptable name
+and prev name = Hashtbl.find prevtable name
 ;;
-
-let change_aux t oldname name =
-  let olds = Hashtbl.find_all t oldname in
-  List.iter
-    (fun s ->
-      Hashtbl.remove t oldname ;
-      Hashtbl.add t name s)
-    olds ;
-  Hashtbl.iter
-    (fun k x ->
-      if !x = oldname then begin
-        x := name
-      end)
-    t
-  
-let change oldname name =
-  change_aux nexttable oldname name ;
-  change_aux prevtable oldname name ;
-  change_aux uptable oldname name
