@@ -7,15 +7,13 @@
 (*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
-(*  $Id: ultra.ml,v 1.10 2005/02/25 17:49:18 maranget Exp $             *)
+(*  $Id: ultra.ml,v 1.13 2006-10-09 08:25:16 maranget Exp $             *)
 (***********************************************************************)
 
 open Tree
 open Htmltext
 open Util
 
-let verbose = ref 0
-
 let same_prop f s =
   try
     let p = Htmltext.get_prop f.nat in
@@ -79,7 +77,7 @@ let rec all_blanks ts i j =
 
 let rec get_same ts i j f = function
   | [] -> ((i,j),f)
-  | ((ii,jj),g)::rem when
+  | ((ii,jj),_)::_ when
        covers i j ii jj &&
        all_blanks ts i (ii-1) &&
        all_blanks ts (jj+1) j -> ((ii,jj),f)
@@ -124,7 +122,7 @@ and correct_cfl_high ts i1 j1 i2 j2 =
 
 let rec mk_cover_one ts i j f = function
   | [] -> (i,j),f
-  | ((ii,jj),g)::rem ->
+  | ((ii,jj),_)::rem ->
       mk_cover_one
         ts
         (correct_cfl_low ts ii jj i j)
@@ -203,7 +201,7 @@ let factorize low high ts =
   let r = do_rec low [] [] in
   let r = group_font ts r in
   let r = extend_neutrals ts r in
-  if r <> [] && !verbose > 1 then begin
+  if r <> [] && !Emisc.verbose > 1 then begin
     Printf.fprintf stderr "Factors in %d %d\n" low high ;
     for i=low to high do
       Pp.tree stderr ts.(i)
@@ -275,7 +273,7 @@ let slen f =
   else
     0) + String.length f.txt + String.length f.ctxt
 
-let order_factors (((i1,j1),f1),c1) (((i2,j2),f2),c2) =
+let order_factors (((_i1,_j1),f1),c1) (((_i2,_j2),f2),c2) =
   if c1 < c2 then true
   else if c1=c2 then
     slen f1 >= slen f2
@@ -286,7 +284,7 @@ let select_factors fs =
   let fs1 = put_conflicts fs in
   let fs2 = biggest fs1 in
   let fs3 = Sort.list order_factors fs2 in
-  if !verbose > 1 then begin
+  if !Emisc.verbose > 1 then begin
     prerr_string "fs1:" ; pfactorc stderr fs1 ;
     prerr_string "fs2:" ; pfactorc stderr fs2 ;
     prerr_string "fs3:" ; pfactorc stderr fs3
@@ -363,7 +361,7 @@ let rec as_long_end p = function
           
 
       
-let bouts p ts =
+let bouts _p ts =
   let bef,rem = as_long is_blank ts in
   let inside,aft = as_long_end is_blank rem in
   bef,inside,aft
@@ -486,7 +484,7 @@ let rec deeper i j ts k =
         if not r then
           ts.(i) <- Node (s,opt true (Array.of_list args) []) ;
         r
-    | t -> again r (i+1) in
+    | _ -> again r (i+1) in
   if again false i then begin
     let ts = as_list i j ts [] in    
     let rs = opt true  (Array.of_list ts) k in
@@ -511,7 +509,7 @@ and trees i j ts k =
               (check_node gs (trees ii jj ts [])
                  (zyva (jj+1) rem k)) in
         let fs = select_factors fs in
-        if !verbose > 1 then begin
+        if !Emisc.verbose > 1 then begin
           prerr_endline "selected" ;
           List.iter
             (fun ((i,j),fs) ->
@@ -527,7 +525,14 @@ and trees i j ts k =
 and opt_onodes ts i = match ts.(i) with
   |  ONode (o,c,args) -> begin match opt false (Array.of_list args) [] with
       | [Node (s,args)] ->
-          ts.(i) <- Node (s,[ONode (o,c,args)])
+	  let s1, s2 = partition_color s in
+	  ts.(i) <-
+	     begin match s1, s2 with
+	     | [],[] -> assert false
+	     | [],s  -> ONode (o,c,[Node (s, args)])
+	     | s,[]  -> Node (s,[ONode (o,c,args)])
+	     | _,_   -> Node (s1, [ONode (o,c,[Node (s2, args)])])
+	     end
       | t ->
           ts.(i) <- ONode (o,c,t)
   end
@@ -568,7 +573,7 @@ let main chan ts =
   let rs =  opt true (Array.of_list (Explode.trees ts)) [] in
   let cf = costs Htmltext.cost rs in
   if compare ci cf < 0 then begin
-    if !verbose > 1 then begin
+    if !Emisc.verbose > 1 then begin
       prerr_endline "*********** Pessimization ***********" ;
       Pp.ptrees stderr ts ;
       prerr_endline "***********   Into        ***********" ;
